#!/bin/ksh

#-------------------------------------------------------------------------
# Special development overrides
#-------------------------------------------------------------------------
developmentOverrideToSkipCdromVerification='no'

#-------------------------------------------------------------------------
# Control flags to disable portions of script execution
#-------------------------------------------------------------------------

skipTsmServerBaseOn3466tsmUpdateCdrom1='no'
skipTsmServerPtf1On3466tsmUpdateCdrom1='no'
skipWebshellPackageOn3466tsmUpdateCdrom1='no'
skipWebclientPackageOn3466tsmUpdateCdrom1='no'
skipTsmSWAon3466tsmUpdateCdrom1='no'
skipTsmClientOn3466tsmUpdateCdrom1='no'
skipTsmDocOn3466tsmUpdateCdrom2='no'

skipBringingServerDown='no'
skipUsrFilesystemAdjustment='no'
skipServerRelicense='no'

#-----------------------------------------------------------------------
#
# (C) COPYRIGHT International Business Machines Corp. 2000
# All Rights Reserved
# Licensed Materials - Property of IBM
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#@ ---------------------------------------------------------------------
#@
#@ Description: This script updates 3466 Cx0 system to TSM Server
#@              level 4.1.1.0 as well as other related resources
#@            
#@              This script requires that EC h27349 be applied. 
#@              It is the invoker's responsibility to verify this
#@              by invoking the 'lsSSlevel' command.
#@
#@              This script requires the system be a 3466 Cx0. This 
#@              is verified by checking that the system is an 
#@              RS/6000 H50.
#@
#@ Special flags
#@
#@ Example usage:
#@
#@   h27608a1.ec -h    : issue help text 
#@   h27608a1.ec       : run all update phases
#@   h27608a1.ec -b    : bypass check for pre-req EC h27349
#@ 
#@ Log Files:
#@ ----------
#@   Log files are kept in the directory /tmp/updateEc_<ec name>. Log files for  
#@   an invocation will be prefixed with the same timestamp. The
#@  time stamp for a file is in the format mmdd_hhmm. Specific log files 
#@
#@   All phases
#@   ----------
#@     -- <mmdd_hhmm>.consoleLog - screen output
#@
#@   TSM / 3466 update phase
#@   -----------------------
#@     -- <mmdd_hhmm>.tsmAixClient.log - output from installation of the AIX client
#@     -- <mmdd_hhmm>.tsmServerBaseInstall.log - output from installation of the TSM Server Base
#@     -- <mmdd_hhmm>.tsmServerPtf1.log - output from installation of the TSM Server PTF 1
#@     -- <mmdd_hhmm>.webclient_package.log - output from installation of the 3466 webclient package
#@     -- <mmdd_hhmm>.webshell_package.log - output from installation of the 3466 webshell package
#@     -- <mmdd_hhmm>.tsmSWA.log - output from copy of the TSM SWA resources
#@     -- <mmdd_hhmm>.tsmDoc.log - output from copy of the TSM Doc resources
#@   

#
# Change History   Date           Description
# --------------   ------------   --------------------------------------
# 10/26/2000       Chris Knapp    Create 
# 04/20/2001       Chris Knapp    add .toc verification
# 08.29.2001       Howard Jiang   Changed x of 3 messages to 1 or 2 of 2
# 09.04.2001       Howard Jiang   Bypassed installation and checking of
#                                 buta AFS/DFS TSM Clients
#
#-------------------------------------------------------------------------

enterOneToContinue ()
{
  echo
  echo "Make selection to continue"
  echo
  sleep 1 # seems to be required, or output gets messed up
  PS3=''
  select userResponse in \
      'Enter 1 followed by the ENTER key to continue

           -- OR --

       Enter CNTL-C to terminate program'
  do
    case $REPLY in
      1) :
         echo 
         echo 'User entered 1, continuing ...........'
         ;;
      *) echo
         echo "User entered: '$REPLY'. The only valid response is 1"
         echo
         echo 'Enter 1 followed by the ENTER key to continue'
         echo 
         echo '---- OR ----'
         echo 
         echo 'Enter CNTL-C to force program termination'
         echo
         ;;
      esac

    if [[ -n $userResponse ]] ; then
      break
    fi
  done

} # end enterOneToContinue


# ----------------------------------------------------------------
# CNTL-C trap routine
# ----------------------------------------------------------------
cntlCtrap ()
{
  echo
  echo "User entered CNTL-C"
  echo

  theCommand='umount /cd'
  echo
  echo "Issuing command to unmount /cd: '$theCommand' (note: may not be mounted)"
  eval $theCommand

  theCommand='rmfs /cd'
  echo
  echo "Issuing command to remove /cd filesystem: '$theCommand' (note: may not exist)"
  eval $theCommand

} # end cntlCtrap

initializeInstallationDirectories ()
{

  # -----------------------------------------------------------------------------------------
 
  tsm_update_cdrom1_altdir='/storsmart/fieldbills/h27608a_cdroms_Cx0_tsm4110/TSM_NSM_ETC_kit_19p1605.cdrom_1of2_19p1606'
  nsmTsm_update_cdrom1_tsmClient="/cd/tsmClient.4.1.0.0.ga"
  nsmTsm_update_cdrom1_tsmServer="/cd/tsmServer.4.1.0.0.ga"
  nsmTsm_update_cdrom1_PTF1="/cd/tsmServerPtf.4.1.1.0"
  nsmTsm_update_cdrom1_webshell_package="/cd/webshell.4.1.0.0"
  nsmTsm_update_cdrom1_webclient_package="/cd/webclients.4.1.0.0"

  targetCopyDir='/usr/nsm.3466/images'
  tsmSWApath='/cd'
  tsmSWAdir='tsmSWA.4.1.1.0'

 
  tsm_update_cdrom2_altdir='/storsmart/fieldbills/h27608a_cdroms_Cx0_tsm4110/TSM_NSM_ETC_kit_19p1605.cdrom_2of2_19p1607'
  tsmDOCpath='/cd'
  tsmDOC='tsmdoc.4.1.0.0'
  oldTsmDOC='tsmdoc.3.7.0.0'
  tsmDOChtmlDir="$tsmDOC/html/aix"
  tsmDOCpdfDir="$tsmDOC/pdf/aix"
  tsmDocLink='/usr/lpp/internet/server_root/pub/adsmdocs'

} # end initializeInstallationDirectories

displayControlVariables ()
{
  # -----------------------------------------------------------
  # display control variables if user has changed any values to skip
  # -----------------------------------------------------------
  if [ "$skipTsmSWAon3466tsmUpdateCdrom1" = 'yes' ] || \
     [ "$skipTsmServerBaseOn3466tsmUpdateCdrom1" = 'yes' ] || \
     [ "$skipTsmServerPtf1On3466tsmUpdateCdrom1" = 'yes' ] || \
     [ "$skipWebshellPackageOn3466tsmUpdateCdrom1" = 'yes' ] || \
     [ "$skipWebclientPackageOn3466tsmUpdateCdrom1" = 'yes' ] || \
     [ "$skipTsmClientOn3466tsmUpdateCdrom1" = 'yes' ] || \
     [ "$skipTsmDocOn3466tsmUpdateCdrom2" = 'yes' ] || \
     [ "$skipBringingServerDown" = 'yes' ] || \
     [ "$skipUsrFilesystemAdjustment" = 'yes' ] || \
     [ "$skipServerRelicense='no'" = 'yes' ]
  then
    echo 
    echo "********************************************************************************"
    echo '*'
    echo '* User has specified to change one or more control variables'
    echo '*'
    echo "********************************************************************************"
    echo
    echo "skipTsmServerBaseOn3466tsmUpdateCdrom1='$skipTsmServerBaseOn3466tsmUpdateCdrom1'"
    echo "skipTsmServerPtf1On3466tsmUpdateCdrom1='$skipTsmServerPtf1On3466tsmUpdateCdrom1'"
    echo "skipWebshellPackageOn3466tsmUpdateCdrom1='$skipWebshellPackageOn3466tsmUpdateCdrom1'"
    echo "skipWebclientPackageOn3466tsmUpdateCdrom1='$skipWebclientPackageOn3466tsmUpdateCdrom1'"
    echo "skipTsmClientOn3466tsmUpdateCdrom1='$skipTsmClientOn3466tsmUpdateCdrom1'"
    echo "skipTsmSWAon3466tsmUpdateCdrom1='$skipTsmSWAon3466tsmUpdateCdrom1'"
    echo
    echo "skipTsmDocOn3466tsmUpdateCdrom2='$skipTsmDocOn3466tsmUpdateCdrom2'"
    echo
    echo "skipBringingServerDown='$skipBringingServerDown'"
    echo "skipUsrFilesystemAdjustment='$skipUsrFilesystemAdjustment'"
    echo "skipServerRelicense='$skipServerRelicense'"
    echo 
    enterOneToContinue
  fi

} # end displayControlVariables

adjustSizeOfUsrFilesystem ()
{
  echo
  echo "INFO ($scriptName): this routine adjusts the size of the /usr filesystem"
  echo "    to accomodate the upgrades"
  echo
  requiredForUpgrade='447992' # actual amount observed

  currentAllocForUsr=`df /usr | grep -v '512-blocks' | awk '{print $2}'`
  currentFreeInUsr=`df /usr | grep -v '512-blocks' | awk '{print $3}'`
  currentUsedInUsr=$(($currentAllocForUsr - $currentFreeInUsr))

  freePPsOnHdisk0=`lspv hdisk0 | grep 'FREE PPs:' | awk '{print $3}'`
  ppSizeInMbForHdisk0=`lspv hdisk0 | grep 'PP SIZE:' | awk '{print $3}'`
  mbFreeOnHdisk0=$(($freePPsOnHdisk0 * $ppSizeInMbForHdisk0))
  availableOnHdisk0=$(($mbFreeOnHdisk0 * 2048))
  totalAvailableOnHdisk0=$(($currentAllocForUsr + $availableOnHdisk0))

  freePPsOnHdisk1=`lspv hdisk1 | grep 'FREE PPs:' | awk '{print $3}'`
  ppSizeInMbForHdisk1=`lspv hdisk1 | grep 'PP SIZE:' | awk '{print $3}'`
  mbFreeOnHdisk1=$(($freePPsOnHdisk1 * $ppSizeInMbForHdisk1))
  availableOnHdisk1=$(($mbFreeOnHdisk1 * 2048))
  totalAvailableOnHdisk1=$(($currentAllocForUsr + $availableOnHdisk1))

  totalRequiredForUpgradePlus0Percent=$(($currentUsedInUsr + $requiredForUpgrade))
  requiredAfterUpgradePlus1Percent=$(($totalRequiredForUpgradePlus0Percent / 100))

  targetSizeAfterUpgradePlus20Percent=$(($totalRequiredForUpgradePlus0Percent + ($requiredAfterUpgradePlus1Percent * 20)))
  targetSizeAfterUpgradePlus10Percent=$(($totalRequiredForUpgradePlus0Percent + ($requiredAfterUpgradePlus1Percent * 10)))
  targetSizeAfterUpgradePlus5Percent=$(($totalRequiredForUpgradePlus0Percent + ($requiredAfterUpgradePlus1Percent * 5)))
  targetSizeAfterUpgradePlus1Percent=$(($totalRequiredForUpgradePlus0Percent + $requiredAfterUpgradePlus1Percent))

# additionalRequiredForPlus20Percent=$(($targetSizeAfterUpgradePlus20Percent - $currentAllocForUsr - $currentFreeInUsr))
# additionalRequiredForPlus10Percent=$(($targetSizeAfterUpgradePlus10Percent - $currentAllocForUsr - $currentFreeInUsr))
# additionalRequiredForPlus5Percent=$(($targetSizeAfterUpgradePlus5Percent - $currentAllocForUsr - $currentFreeInUsr))
# additionalRequiredForPlus1Percent=$(($targetSizeAfterUpgradePlus1Percent - $currentAllocForUsr - $currentFreeInUsr))

  echo 
  echo "currentAllocForUsr $currentAllocForUsr"
  echo "currentFreeInUsr $currentFreeInUsr"
  echo "currentUsedInUsr $currentUsedInUsr"
  echo
  echo "ppSizeInMbForHdisk0 $ppSizeInMbForHdisk0"
  echo "mbFreeOnHdisk0 $mbFreeOnHdisk0"
  echo "freePPsOnHdisk0 $freePPsOnHdisk0"
  echo "availableOnHdisk0 $availableOnHdisk0"
  echo "totalAvailableOnHdisk0 $totalAvailableOnHdisk0"
  echo
  echo "ppSizeInMbForHdisk1 $ppSizeInMbForHdisk1"
  echo "mbFreeOnHdisk1 $mbFreeOnHdisk1"
  echo "freePPsOnHdisk1 $freePPsOnHdisk1"
  echo "availableOnHdisk1 $availableOnHdisk1"
  echo "totalAvailableOnHdisk1 $totalAvailableOnHdisk1"
  echo 
  echo "requiredForUpgrade $requiredForUpgrade"
  echo "requiredAfterUpgradePlus1Percent $requiredAfterUpgradePlus1Percent"
  echo
  echo "targetSizeAfterUpgradePlus20Percent $targetSizeAfterUpgradePlus20Percent"
  echo "targetSizeAfterUpgradePlus10Percent $targetSizeAfterUpgradePlus10Percent"
  echo "targetSizeAfterUpgradePlus5Percent $targetSizeAfterUpgradePlus5Percent"
  echo "targetSizeAfterUpgradePlus1Percent $targetSizeAfterUpgradePlus1Percent"
  echo "totalRequiredForUpgradePlus0Percent $totalRequiredForUpgradePlus0Percent"

# echo
# echo "additionalRequiredForPlus20Percent $additionalRequiredForPlus20Percent"
# echo "additionalRequiredForPlus10Percent $additionalRequiredForPlus10Percent"
# echo "additionalRequiredForPlus5Percent $additionalRequiredForPlus5Percent"
# echo "additionalRequiredForPlus1Percent $additionalRequiredForPlus1Percent"

  echo
  if (( $currentAllocForUsr >= $targetSizeAfterUpgradePlus10Percent))
  then
    echo "INFO ($scriptName): The number of 512 blocks for the /usr filesystem: $currentAllocForUsr"
    echo " is already sufficient to accomodate the upgrade" 
    theCommand='space currently OK'
  else
    if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus20Percent))
    then
      echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 20% reserve"
      targetSize=$targetSizeAfterUpgradePlus20Percent
      theCommand="chfs -a size=$targetSize /usr"
    else
      echo
      echo "Insufficient space on hdisk0 to increase the size of /usr with 20% reserve"
      echo
      if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus10Percent))
      then
        echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 10% reserve"
        targetSize=$targetSizeAfterUpgradePlus10Percent
        theCommand="chfs -a size=$targetSize /usr"
      else
        echo
        echo "Insufficient space on hdisk0 to increase the size of /usr with 10% reserve"
        echo
        if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus5Percent))
        then
          echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 5% reserve"
          targetSize=$targetSizeAfterUpgradePlus5Percent
          theCommand="chfs -a size=$targetSize /usr"
        else
          echo
          echo "Insufficient space on hdisk0 to increase the size of /usr with 5% reserve"
          echo
          if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus1Percent))
          then
            echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 1% reserve"
            targetSize=$targetSizeAfterUpgradePlus1Percent
            theCommand="chfs -a size=$targetSize /usr"
          else
            echo "Insufficient space on hdisk0 to increase the size of /usr with 1% reserve"
            theCommand='insufficient space'
          fi
        fi
      fi
    fi
  fi

  echo
  if [ "$theCommand" = 'insufficient space' ]
  then
    echo "NSM-95301-E: ERROR ($scriptName) - there is insufficient space available to increase"
    echo " the size of the /usr filesystem to accomodate this upgrade. Corrective"
    echo " action is required. Contact IBM support for assistance"
    echo
  elif [ "$theCommand" = 'space currently OK' ]
  then
    : 
  else
    if (( $totalAvailableOnHdisk1 >= $targetSize))
    then
      echo
      echo "INFO ($scriptName): issuing command: $theCommand"
      eval $theCommand
      chfsRc=$?
      if (( $chfsRc != 0 ))
      then
        echo "NSM-95302-E: ERROR ($scriptName) - failure changing the size of /usr filestystem"
        echo "  corrective action required. Contact IBM Support"
        echo 
        echo "exiting ...."
        echo 
        exit 1
      fi
    else
      echo "NSM-95303-E: ERROR ($scriptName) - the amount of space on hdisk1: $totalAvailableOnHdisk1 512 byte blocks"
      echo " is not sufficient for the required allocation of $targetSize 512 byte blocks."
      echo " The amount of space on hdisk0: $totalAvailableOnHdisk0 512 byte blocks is sufficient"
      echo " Disks hdisk0 and hdisk1 should always have an equivalent amount of free space"
      echo " Corrective action required. Contact IBM Support"
      echo 
      echo "exiting ...."
      echo
      exit 1
    fi
  fi

} # end adjustSizeOfUsrFilesystem

checkFileSystemUtilization () # this routine is not used due to space concerns for 4.5GB operating system disks
{
  fileSystemList='/:50 /usr:80 /tmp:50 /var:50'
  allFilesystemsExist='yes'
  allFilesystemsUtilzationOk='yes'
  echo
  for currentFileSystemCombo in $fileSystemList
  do

    # --------------------------------------------
    # Parse information for the current filesystem
    # --------------------------------------------
    currentFileSystem=`echo $currentFileSystemCombo | cut -f1 '-d':'`
    currentFileSystemThreshHold=`echo $currentFileSystemCombo | cut -f2 '-d':'`

    # -------------------------------------
    # verify the filesystem exists
    # -------------------------------------
    df $currentFileSystem >/dev/null 2>&1
    dfRc=$?
    if (( $dfRc > 0 ))
    then
      allFilesystemsExist='no'
      echo "NSM-95304-E: ERROR: ($scriptName) - The filesystem '$currentFileSystem' does not exist"
    else
      # -------------------------------------
      # verify filesystem utilization
      # -------------------------------------
      fileSystemDfString=`df $currentFileSystem | grep -v 'Filesystem'`
      fileSystemUtilization=`echo $fileSystemDfString | awk '{print $4}' | cut -f1 -d'%'`
      if (( $fileSystemUtilization > $currentFileSystemThreshHold ))
      then
        echo
        echo "INFO: ($scriptName) - The utilization for filesystem '$currentFileSystem' is $fileSystemUtilization%"
        echo "  This is greater than the maximum utilization of $currentFileSystemThreshHold"
        fileSystemSize512KbBlocks=`echo $fileSystemDfString | awk '{print $2}'`
        fileSystemFree512KbBlocks=`echo $fileSystemDfString | awk '{print $3}'`
        fileSystemUsed512KbBlocks=$(($fileSystemSize512KbBlocks - $fileSystemFree512KbBlocks))
        echo "  The current filesystem size in 512KB blocks is: $fileSystemSize512KbBlocks"
        echo "  The number of free 512KB blocks is: $fileSystemFree512KbBlocks"
        echo "  The number of used 512KB blocks is: $fileSystemUsed512KbBlocks"
        numberOf512BlocksToAdd=$(( (($fileSystemUsed512KbBlocks*100)/$currentFileSystemThreshHold) - $fileSystemSize512KbBlocks + 1 ))
        echo "  The number of 512KB blocks to add is: $numberOf512BlocksToAdd"
        newTotal512Blocks=$(($fileSystemSize512KbBlocks + $numberOf512BlocksToAdd))
        echo "  The new filesystem size in blocks will be: $newTotal512Blocks"
        echo 
        theCommand="chfs -a size=+$numberOf512BlocksToAdd $currentFileSystem"
        echo "INFO: ($scriptName) - issuing command: '$theCommand'"
        eval $theCommand
        theCommandRc=$?
        if (( $theCommandRc > 0 ))
        then
          echo 
          echo "NSM-95305-E: ERROR: ($scriptName) - error trying to change the file size for filesystem: $currentFileSystem"
          allFilesystemsUtilzationOk='no'
        fi
        echo
      else
        echo "INFO: ($scriptName) - The utilization for filesystem '$currentFileSystem' is $fileSystemUtilization%"
        echo "  this is less than the maximum threshold of $currentFileSystemThreshHold%"
      fi
    fi
  done

  if [ "$allFilesystemsExist" = 'no' ]
  then
    echo
    echo "NSM-95306-E: ERROR: ($scriptName) - EXITING, one or more required filesystems do not exist"
    exit 1
  fi

  if [ "$allFilesystemsUtilzationOk" = 'no' ]
  then
    echo
    echo "NSM-95307-E: ERROR: ($scriptName) - EXITING, one or more filesystems exceed the maximum filesystem utilization"
    exit 1
  fi
} # end checkFileSystemUtilization

verifyProperCdromMounted ()
{

# ------------------------------------------------------------------------
# verify the proper cdrom is mounted
# ------------------------------------------------------------------------
doneVerifyingCdrom='no'
while [ "$doneVerifyingCdrom" = 'no' ]
do

  checkForCorrectCdrom=''

  echo
  echo "INFO: ($scriptName) - insert CD with title $cdromLabel"
  echo "                    into the CDrom reader."
  echo
  enterOneToContinue

  if [ "$developmentOverrideToSkipCdromVerification" = 'yes' ]
  then
    echo
    echo "INFO: ($scriptName) - skipping cdrom verification since developmentOverrideToSkipCdromVerification=$developmentOverrideToSkipCdromVerification"
    return
  fi

  if [ "$mountTarget" = '/dev/cd0' ]
  then
    theCommand='mount /cd'
    echo
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
  else
    theCommand="mount -n $nfsHostForH27608 $mountTarget /cd"
    echo
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
  fi

  checkIfCdromMounted=`df | grep '/cd' | awk '{print $7}'`
  if [ "$checkIfCdromMounted" = '/cd' ]
  then
    echo
    df /cd
    echo 
  else
    checkForCorrectCdrom='notMounted'
    echo "NSM-95308-E: ERROR: ($scriptName) - unable to mount $cdromLabel"
  fi 

  if [ "$checkForCorrectCdrom" != 'problem' ] && [ "$checkForCorrectCdrom" != 'notMounted' ]
  then
    echo
    echo "INFO: ($scriptName) - verifying that the cdrom is $cdromLabel"
    echo 
    theCommand="ls $fileUniqueToThisCdrom"'*'
    echo 
    echo "INFO: ($scriptName) - Issuing command: $theCommand"
    eval $theCommand
    theRc=$?
    if [ $theRc = 0 ]
    then
      checkForCorrectCdrom='ok'
    else
      checkForCorrectCdrom='problem'
    fi
  fi

  if [ "$checkForCorrectCdrom" = 'notMounted' ]
  then
    echo
    echo '**********************************************************************'
    echo '*'
    echo "* NSM-95309-E: ERROR: ($scriptName) - the cdrom does not appear to be mounted"
    echo '*'
    echo '**********************************************************************'
    echo 

  elif [ "$checkForCorrectCdrom" = 'problem' ]
  then
    echo
    echo '**********************************************************************'
    echo '*'
    echo "* NSM-95310-E: ERROR: ($scriptName) - unable to verify that the $cdromLabel"
    echo "*   is inserted in the cdrom reader. Check the following:"
    echo '*'
    echo "* Verification file is: $fileUniqueToThisCdrom'*'"
    echo '*'
    echo "*  It is likely that an incorrect cdrom is mounted"
    echo '*'
    echo '**********************************************************************'
    echo 
    theCommand='unmount /cd'
    echo 
    echo "Issuing command: $theCommand"
    echo
    eval $theCommand
    theCommandRc=$?
    echo
    if (( $theCommandRc == 0 ))
    then
      echo "The incorrect cdrom was unmounted"
    else
      echo "NSM-95311-E: ERROR unmounting cdrom. Manual intervention required"
      echo 
      echo '  1) determine why cdrom cannot be unmounted'
      echo '     one likely cause is the directory is cded into'
      echo '     from some other window'
      echo "  2) manually issue the command: '$theCommand'"
    fi

  else
    doneVerifyingCdrom='yes'
    echo
    echo "INFO: ($scriptName) - verification that $cdromLabel"
    echo " is inserted in Cdrom drive is successfull. Continuing ...."
  fi

done 

} # end verifyProperCdromMounted

unmountCd ()
{
  theCommand='umount /cd'
  echo
  echo "INFO: ($scriptName) - issuing command: $theCommand"
  eval $theCommand
  umountRc=$? 
  if [ $umountRc != 0 ]
  then
    echo
    echo "NSM-95312-E: ERROR: ($scriptName) - unable to unmount $cdromLabel"
    echo 
    echo "It is required that you determine the reason the cdrom cannot be unmounted."
    echo "A typical cause is that someone has issued the cd command to change to this"
    echo "directory. This prevents the unmount from working."
    echo 
    echo "Once you have removed the cause for the unmount failure, the /cd filesystem"
    echo "must be unmounted from another window with the command 'unmount /cd'"
    echo 
    echo "Indicate what action to take"
    echo
    sleep 1
    PS3="Enter choice: "
    select userResponse in \
        'continue - the /cd filesystem is unmounted' \
        'exit - unable to unmount the /cd filesystem'
    do
      case $REPLY in
        1) echo "User specified option 1: 'continue - the /cd filesystem is unmounted'"
           checkIfCdromMounted=`df | grep '/cd' | awk '{print $7}'`
           if [ "$checkIfCdromMounted" = '/cd' ]
           then
             echo
             echo "NSM-95313-E: ERROR: ($scriptName) - the /cd rom filesystem is still mounted"
             echo 
             echo 'Your response that the cdrom filesystem was unmounted is incorrect'
             echo
             theCommand='df'
             echo "Issuing command: $theCommand"
             echo 
             eval $theCommand
             echo 
             echo "Exiting .................."
             echo 
             exit 1
           fi
           ;;
        2) echo
           echo "User specified option 2: 'exit - unable to unmount the /cd filesystem"
           echo "exiting ......"
           exit
           ;;
        *) echo 'Invalid Selection.'
           ;;
      esac
 
      if [[ -n $userResponse ]] ; then
        break
      fi
    done
  fi

} # end unmountCd

verifyToc ()
{
  listOfDirectoryWhichMustContainToc=$*
  echo
  tocValidationPassed='yes'
  for currentDirectory in $listOfDirectoryWhichMustContainToc
  do
    if [ -d $currentDirectory ]
    then
      cd $currentDirectory >/dev/null 2>&1
      if [ -f .toc ]
      then
        echo
        theCommand="installp -ld `pwd`"
        echo "INFO ($scriptName) -  Issuing command: '$theCommand' to verify .toc file integrity"
        echo "   Note: the output is redirected to /dev/null"
        echo
        eval $theCommand >/dev/null 2>&1
        installpRc=$?
        if (( $installpRc == 0))
        then
          :
        else
          tocValidationPassed='no'
          echo
          echo "NSM-95314-E: ERROR ($scriptName) - error from installp using the '.toc' file to list the"
          echo "  directory contents. The return code from installp is: $installpRc"
        fi
      else
        tocValidationPassed='no'
        echo
        echo "NSM-95315-E: ERROR ($scriptName) - the '.toc' file not found in the current directory"
      fi
      cd - >/dev/null 2>&1
    else
      tocValidationPassed='no'
      echo
      echo "NSM-95316-E: ERROR ($scriptName) - the directory '$currentDirectory' does not exist"
      echo "   This is a directory that was specified to contain a '.toc' file"
    fi
  done
  echo
  if [ "$tocValidationPassed" = 'yes' ]
  then
    echo
    echo "INFO ($scriptName) - all '.toc' validation passed for current cdrom"
  else
    echo
    echo "NSM-95317-E: ERROR ($scriptName) - one or more '.toc' validation errors occured for this cdrom"
    echo "  An I/O error is suspected on the Cdrom. Call IBM support for assistance"
    echo
    unmountCd
    echo
    echo "exiting ...."
    echo
    exit 1
  fi
} # end verifyToc

checkFilesetLevel ()
{

# --------------------------------------------------
# Capture positional parms
# --------------------------------------------------
fileset="$1"
requiredLevel="$2"
echo ${requiredLevel} | sed -e "s/\./ /g" | read Level1Required Level2Required Level3Required Level4Required Level5Required
echo
echo "Checking for fileset: '$fileset' at level: '$requiredLevel' or higher"


# --------------------------------------------------
# Capture and parse information for the fileset to
# determine if it is installed and at what level
# --------------------------------------------------
LSLPP=`lslpp -L $fileset 2>/dev/null`
lslppRc=$?
if [ $lslppRc = 0 ]
then
  filesetFound='no'
  filesetLevelFound='no'
  for curWord in $LSLPP
  do
    if [ "$filesetFound" = 'yes' ]
    then
      filesetLevel="$curWord"
      filesetLevelFound='yes'
      break
    elif [ "$curWord" = "$fileset" ]
    then
      filesetFound='yes'
    fi
  done
fi
# --------------------------------------------------
# If fileset was found determine if it is installed
# at the required level or higher
# --------------------------------------------------
returnString='undetermined' # set return code, will be changed to another value when state is determined
if [ "$filesetFound" = 'yes' ] && [ "$filesetLevelFound" = 'yes' ]
then
  echo "Fileset: '$fileset' found at level: '$filesetLevel'"

  echo ${filesetLevel} | sed -e "s/\./ /g" | read Level1captured Level2captured Level3captured Level4captured Level5captured

  # -------------------------------------------
  # Determine if level 1 is at a suffcicient level
  # if higher than the required level 1 checking is done
  # if lower than the required level 1 checking is done
  # -------------------------------------------
  if [ $Level1captured -gt $Level1Required ] # check level 1
  then
    returnString='ok' # level is ok, all done checking
  else
    if [ $Level1captured -lt $Level1Required ] # check level 1
    then
      returnString='mismatch at level 1' # level 1 is not ok, all done checking
    fi
  fi

  # -------------------------------------------
  # Determine if level 2 is at a suffcicient level
  # if higher than the required level 2 checking is done
  # if lower than the required level 2 checking is done
  # -------------------------------------------
  if [ "$Level2captured" = '' ]   
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level2captured -gt $Level2Required ] # check level 2
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level2captured -lt $Level2Required ] # check level 2
      then
        returnString='mismatch at level 2' # level 2 is not ok, all done checking
      fi
    fi
  fi

  # -------------------------------------------
  # Determine if level 3 is at a suffcicient level
  # if higher than the required level 3 checking is done
  # if lower than the required level 3 checking is done
  # -------------------------------------------
  if [ "$Level3captured" = '' ]
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level3captured -gt $Level3Required ] # check level 3
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level3captured -lt $Level3Required ] # check level 3
      then
        returnString='mismatch at level 3' # level 3 is not ok, all done checking
      fi
    fi
  fi

  # -------------------------------------------
  # Determine if level 4 is at a suffcicient level
  # if higher than the required level 4 checking is done
  # if lower than the required level 4 checking is done
  # -------------------------------------------
  if [ "$Level4captured" = '' ]
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level4captured -gt $Level4Required ] # check level 4
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level4captured -lt $Level4Required ] # check level 4
      then
        returnString='mismatch at level 4' # level 4 is not ok, all done checking
      fi
    fi
  fi

  # -------------------------------------------
  # Determine if level 5 is at a suffcicient level
  # if higher than the required level 5 checking is done
  # if lower than the required level 5 checking is done
  # -------------------------------------------
  if [ "$Level5captured" = '' ]
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level5captured -gt $Level5Required ] # check level 5
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level5captured -lt $Level5Required ] # check level 5
      then
        returnString='mismatch at level 5' # level 5 is not ok, all done checking
      fi
    fi
  fi

  if [ "$returnString" = 'undetermined' ]
  then
    returnString='ok' # level is ok, all done checking
  fi

else
  echo "Fileset: '$fileset' NOT FOUND"
  returnString='lpp not found' # lpp not found
fi

if [ "$returnString" = 'ok' ]
then
  returnCode=0
elif [ "$returnString" = 'lpp not found' ]
then
  returnCode=2
else
  returnCode=1
fi

return $returnCode

} # end checkFilesetLevel

updateAll  ()
{
  installDirectory="$1"
  estimatedTime="$2"  # estimated installation time in minutes
  installRatherThanUpdate=$3

  # ------------------------------------------------------------------------
  # do the updates
  # ------------------------------------------------------------------------
  if [ "$mountTarget" != '/dev/cd0' ]
  then
    echo
#   theCommand="inutoc $installDirectory"
#   echo "INFO: ($scriptName) - issuing command: $theCommand"
#   echo 
#   eval $theCommand
#   echo
  fi
  echo
  echo "INFO: ($scriptName) - the installation is logged in: $logFileName"
  echo "INFO: ($scriptName) - the installation log can be monitored with the command:"
  echo "                    'tail -f $logFileName'"
  echo "                    CNTL-C to terminate the tail command" 
  echo
  echo "INFO: ($scriptName) - estimated installation time is: $estimatedTime minutes"
  echo
  if [ "$installRatherThanUpdate" = '' ]
  then


    theCommand="/usr/lib/instl/sm_inst installp_cmd -a -d '$installDirectory' -f '_update_all' '-c' '-N' '-g' '-X' '-S'"

    # -----------------------------------------------------------------------------
    # This hardcoded update was put in because if AIX DCE/DFS 3.1 is installed, the
    # buta dfs TSM client will fail, causing the whole script to fail.  So the AFS
    # and DFS buta clients are excluded in the install list below

    if [ "$installDirectory" = "$nsmTsm_update_cdrom1_tsmClient" ]
    then
      theCommand="/usr/lib/instl/sm_inst installp_cmd -a -d '$installDirectory' -f 'tivoli.tsm.client.afs.aix43.32bit  ALL  @@tivoli.tsm.client.afs.aix43.32bit _all_filesets,tivoli.tsm.client.api.aix43.32bit  ALL  @@tivoli.tsm.client.api.aix43.32bit _all_filesets,tivoli.tsm.client.api.aix43.64bit  ALL  @@tivoli.tsm.client.api.aix43.64bit _all_filesets,tivoli.tsm.client.ba.aix43.32bit  ALL  @@tivoli.tsm.client.ba.aix43.32bit _all_filesets,tivoli.tsm.client.dfs.aix43.32bit  ALL  @@tivoli.tsm.client.dfs.aix43.32bit _all_filesets,tivoli.tsm.client.hsm.jfs.aix43.32bit  ALL  @@tivoli.tsm.client.hsm.jfs.aix43.32bit _all_filesets,tivoli.tsm.client.image.aix43.32bit  ALL  @@tivoli.tsm.client.image.aix43.32bit _all_filesets,tivoli.tsm.client.web.aix43.32bit  ALL  @@tivoli.tsm.client.web.aix43.32bit _all_filesets'  '-c' '-N' '-g' '-X'"
    fi

    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): before installp command: `date`" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): Issuing command $theCommand"  | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    eval $theCommand >> $logFileName 2>&1
    installp_cmd_Rc=$?
    echo | tee -a $logFileName 2>&1
    echo "The installp_cmd return code is: $installp_cmd_Rc" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): after installp command: `date`" | tee -a $logFileName 2>&1
  else
    theCommand=" /usr/lib/instl/sm_inst installp_cmd -a -Q -d '$installDirectory' -f '_all_latest'  '-c' '-N' '-g' '-X' '-G'"
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): before installp command: `date`" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): Issuing command $theCommand"  | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    eval $theCommand >> $logFileName 2>&1
    installp_cmd_Rc=$?
    echo | tee -a $logFileName 2>&1
    echo "The installp_cmd return code is: $installp_cmd_Rc" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): after installp command: `date`" | tee -a $logFileName 2>&1
  fi

  echo
  echo "INFO: ($scriptName) OSlevel after installing $currentDirectory is: `oslevel`"

} # end updateAll

updateTsmDoc ()
{

  # -----------------------------------------------------
  # delete the old documentation
  # -----------------------------------------------------
  oldDir="$targetCopyDir/$oldTsmDOC"
  echo
  if [ -d "$oldDir" ]
  then
    echo "INFO ($scriptName): deleting old TSM documentation directory - $oldDir"
    theCommand="rm -r $oldDir"
    echo 
    echo "Issuing command: $theCommand"
    echo 
    eval $theCommand
  else
    echo "INFO ($scriptName): old TSM documentation directory - $oldDir does not exist"
  fi
  echo

  # -----------------------------------------------------
  # copy the TSM AIX html documentation"
  # -----------------------------------------------------
  echo
  echo "INFO ($scriptName): Copying images for the TSM AIX html documentation"
  sourceDir="$tsmDOCpath/$tsmDOChtmlDir"
  targetDir="$targetCopyDir/$tsmDOChtmlDir"
  if [ -d $targetDir ]
  then
    echo
    echo "INFO ($scriptName): The target directory: $targetDir already exists"
  else
    theCommand="mkdir -p $targetDir"
    echo
    echo "INFO ($scriptName): Issuing command: $theCommand"
    eval $theCommand
    mkdirRc=$?
    if [ $mkdirRc != 0 ]
    then
      echo
      echo "NSM-95318-E: ERROR ($scriptName) - error making target directory for the TSM AIX html documentation"
      echo
      echo "exiting ....."
      echo 
      exit 1
    fi
  fi
  
  theCommand='cp -R '"$sourceDir/"'*'" $targetDir"
  echo
  echo "INFO ($scriptName): Issuing command: $theCommand"
  echo
  echo "  the estimated time for the copy is: 1 minute or less"
  echo
  time eval $theCommand
  cpRc=$?
  if [ $cpRc != 0 ]
  then
    echo
    echo "NSM-95319-E: ERROR ($scriptName) - error copying secure TSM AIX html documentation"
    echo
    echo "exiting ....."
    echo 
    exit 1
  fi
  
  # -----------------------------------------------------
  # copy the TSM AIX pdf documentation"
  # -----------------------------------------------------
  echo
  echo "INFO ($scriptName): Copying images for the TSM pdf html documentation"
  sourceDir="$tsmDOCpath/$tsmDOCpdfDir"
  targetDir="$targetCopyDir/$tsmDOCpdfDir"
  if [ -d $targetDir ]
  then
    echo
    echo "INFO ($scriptName): The target directory: $targetDir already exists"
  else
    theCommand="mkdir -p $targetDir"
    echo
    echo "INFO ($scriptName): Issuing command: $theCommand"
    eval $theCommand
    mkdirRc=$?
    if [ $mkdirRc != 0 ]
    then
      echo
      echo "NSM-95320-E: ERROR ($scriptName) - error making target directory for the TSM pdf documentation"
      echo
      echo "exiting ....."
      echo 
      exit 1
    fi
  fi
  
  theCommand='cp -R '"$sourceDir/"'*'" $targetDir"
  echo
  echo "INFO ($scriptName): Issuing command: $theCommand"
  echo
  echo "  the estimated time for the copy is: 1 minute or less"
  echo
  time eval $theCommand
  cpRc=$?
  if [ $cpRc != 0 ]
  then
    echo
    echo "NSM-95321-E: ERROR ($scriptName) - error copying secure TSM AIX html documentation"
    echo
    echo "exiting ....."
    echo 
    exit 1
  fi
  
  # --------------------------------------------------
  # set symbolic link for TSM documentation
  # --------------------------------------------------
  theCommand="rm $tsmDocLink"
  echo
  echo "INFO ($scriptName): Remove symbolic link in case it already exists"
  echo "INFO ($scriptName): Issuing command: $theCommand"
  eval $theCommand
  
  theCommand="ln -sf $targetCopyDir/$tsmDOC $tsmDocLink"
  echo
  echo "INFO ($scriptName): Inititalize symbolic link"
  echo "INFO ($scriptName): Issuing command: $theCommand"
  eval $theCommand
  linkRc=$?
  if [ $linkRc != 0 ]
  then
    echo
    echo "NSM-95322-E: ERROR ($scriptName) - error initializing the symbolic link"
    echo
    echo "exiting ....."
    echo 
    exit 1
  fi
  
  } # end updateTsmDoc

updateSwaDoc ()
{

  # -----------------------------------------------------
  # copy the secure TSM web ADMIN client
  # -----------------------------------------------------
  echo 
  echo "INFO ($scriptName): Copying images for the secure TSM web admin client"
  sourceDir="$tsmSWApath/$tsmSWAdir"
  targetDir="$targetCopyDir/$tsmSWAdir"
  if [ -d $targetDir ]
  then
    echo
    echo "INFO ($scriptName): The target directory: $targetDir already exists"
  else
    theCommand="mkdir -p $targetDir"
    echo
    echo "INFO ($scriptName): Issuing command: $theCommand"
    eval $theCommand
    mkdirRc=$?
    if [ $mkdirRc != 0 ]
    then
      echo
      echo "NSM-95323-E: ERROR ($scriptName) - error making target directory for the secure TSM web admin client"
      echo 
      echo "exiting ....."
      echo 
      exit 1
    fi
  fi
  
  theCommand='cp -R '"$sourceDir/"'*'" $targetDir"
  echo
  echo "INFO ($scriptName): Issuing command: $theCommand"
  echo
  echo "  the estimated time for the copy is: 1 minute or less"
  echo
  time eval $theCommand
  cpRc=$?
  if [ $cpRc != 0 ]
  then
    echo
    echo "NSM-95324-E: ERROR ($scriptName) - error copying secure TSM web admin client"
    echo 
    echo "exiting ....."
    echo 
    exit 1
  fi

} # end updateSwaDoc

updateSoftware ()
{

  # --------------------------------------------------------------------------
  # Determine if 3466/TSM update cdrom  1  should be mounted
  # --------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  if [ "$skipTsmClientOn3466tsmUpdateCdrom1" = 'yes' ] && \
     [ "$skipTsmServerBaseOn3466tsmUpdateCdrom1" = 'yes' ] && \
     [ "$skipTsmServerPtf1On3466tsmUpdateCdrom1" = 'yes' ] && \
     [ "$skipWebshellPackageOn3466tsmUpdateCdrom1" = 'yes' ] && \
     [ "$skipWebclientPackageOn3466tsmUpdateCdrom1" = 'yes' ] && \
     [ "$skipTsmSWAon3466tsmUpdateCdrom1" = 'yes' ]
  then
    echo 
    echo "INFO ($scriptName) - skipping mounting / processing 3466/TSM Update cdrom 1 since:"
    echo 
    echo "  skipTsmClientOn3466tsmUpdateCdrom1=$skipTsmClientOn3466tsmUpdateCdrom1"
    echo "  skipTsmServerBaseOn3466tsmUpdateCdrom1=$skipTsmServerBaseOn3466tsmUpdateCdrom1"
    echo "  skipTsmServerPtf1On3466tsmUpdateCdrom1=$skipTsmServerPtf1On3466tsmUpdateCdrom1"
    echo "  skipWebshellPackageOn3466tsmUpdateCdrom1=$skipWebshellPackageOn3466tsmUpdateCdrom1"
    echo "  skipWebclientPackageOn3466tsmUpdateCdrom1=$skipWebclientPackageOn3466tsmUpdateCdrom1"
    echo "  skipTsmSWAon3466tsmUpdateCdrom1=$skipTsmSWAon3466tsmUpdateCdrom1"
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that 3466/TSM update cdrom 1 is mounted
    # ------------------------------------------------------------------------
    fileUniqueToThisCdrom="/cd/tsmClient.4.1.0.0.ga/tivoli.tsm.client.afs.aix43.32bit"
    cdromLabel="3466 and TSM Update CD-ROM, EC: H27608 CD 1 of 2"
    if [ "$nfsHostForH27608" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$tsm_update_cdrom1_altdir"
    fi

    verifyProperCdromMounted
    verifyToc '/cd/tsmClient.4.1.0.0.ga'  '/cd/tsmServer.4.1.0.0.ga'  '/cd/webclients.4.1.0.0'   '/cd/tsmServerPtf.4.1.1.0'  '/cd/webshell.4.1.0.0'

    # ------------------------------------------------------------------------
    # update TSM Aix Client
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipTsmClientOn3466tsmUpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for TSM Aix Client on 3466 Update Cdrom 1 of 2 since:"
      echo 
      echo "  skipTsmClientOn3466tsmUpdateCdrom1=$skipTsmClientOn3466tsmUpdateCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.tsmAixClient.log"
      updateAll "$nsmTsm_update_cdrom1_tsmClient" '2'
  
      # ------------------------------------------------------
      # verify Tivoli AIX client at proper level
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      reqLevel='4.1.0.0'

      checkFilesetLevel tivoli.tsm.client.afs.aix43.32bit $reqLevel
      checkFilesetLevelRc1=$?; 

      checkFilesetLevel tivoli.tsm.client.api.aix43.32bit $reqLevel
      checkFilesetLevelRc2=$?; 

      checkFilesetLevel tivoli.tsm.client.api.aix43.64bit $reqLevel
      checkFilesetLevelRc3=$?; 

      checkFilesetLevel tivoli.tsm.client.ba.aix43.32bit.base $reqLevel
      checkFilesetLevelRc4=$?; 

      checkFilesetLevel tivoli.tsm.client.ba.aix43.32bit.common $reqLevel
      checkFilesetLevelRc5=$?; 

#      checkFilesetLevel tivoli.tsm.client.butaafs.aix43.32bit $reqLevel
      checkFilesetLevelRc6=0; 

#      checkFilesetLevel tivoli.tsm.client.butadfs.aix43.32bit $reqLevel
      checkFilesetLevelRc7=0; 

      checkFilesetLevel tivoli.tsm.client.dfs.aix43.32bit $reqLevel
      checkFilesetLevelRc8=$?; 

      checkFilesetLevel tivoli.tsm.client.image.aix43.32bit $reqLevel
      checkFilesetLevelRc9=$?; 

      checkFilesetLevel tivoli.tsm.client.web.aix43.32bit $reqLevel
      checkFilesetLevelRc10=$?; 

      if (( $checkFilesetLevelRc1 == 0 )) && \
         (( $checkFilesetLevelRc2 == 0 )) && \
         (( $checkFilesetLevelRc3 == 0 )) && \
         (( $checkFilesetLevelRc4 == 0 )) && \
         (( $checkFilesetLevelRc5 == 0 )) && \
         (( $checkFilesetLevelRc6 == 0 )) && \
         (( $checkFilesetLevelRc7 == 0 )) && \
         (( $checkFilesetLevelRc8 == 0 )) && \
         (( $checkFilesetLevelRc9 == 0 )) && \
         (( $checkFilesetLevelRc10 == 0 ))
      then
        successUpdateingTsmAixClient='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade  for TSM AIX Client was successfull"  
      else
        echo
        echo "NSM-95342-E: ERROR verifying TSM AIX client installation. The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic data in file: /tmp/updateEc_""$sciptName/<mmdd_hhmm>.tsmAixClient.log"
        echo
        theCommand='lslpp -l | grep tivoli.tsm.client'
        echo "Issuing command: $theCommand"
        echo 
        eval $theCommand
        echo
        echo "  This script will exit."
        enterOneToContinue
        echo "  Exiting ....."
        unmountCd
        exit 1
        successUpdateingTsmAixClient='no'
      fi
    fi

    # ------------------------------------------------------------------------
    # update TSM Server Base Install
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipTsmServerBaseOn3466tsmUpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for TSM Server Base Install on 3466 Update Cdrom 1 of 2 since:"
      echo 
      echo "  skipTsmServerBaseOn3466tsmUpdateCdrom1=$skipTsmServerBaseOn3466tsmUpdateCdrom1"
      echo
      enterOneToContinue
    else

      # -------------------------------------------------------------------------------
      # Initialize list of filesets:fileset levels that may need to be deleted
      # The fileset is added to the list of filesets to be deleted if it is
      # currently installed at a level lower than specified
      # -------------------------------------------------------------------------------
      fileSetsToDeleteCombo=''
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.devices.acsls:4.1.1.0"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.devices.rte:9.9.9.9"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.license.rte:4.1.0.0"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.msg.en_US.devices:4.1.0.0"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.msg.en_US.server:4.1.1.0"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.msg.en_US.webhelp:4.1.1.0"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.server.com:4.1.1.0"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.server.rte42:9.9.9.9"
      fileSetsToDeleteCombo="$fileSetsToDeleteCombo tivoli.tsm.server.webadmin:4.1.1.0"

      # -------------------------------------------------------------------------------
      # build the list of filesets to delete
      # -------------------------------------------------------------------------------
      echo
      echo "INFO ($scriptName) - check for filesets that need to be deleted"
      fileSetsToPreDelete=''
      echo
      for currentFileSetCombo in $fileSetsToDeleteCombo
      do
        fileSetToDelete=`echo $currentFileSetCombo | cut -f1 -d':'`
        fileSetLevelToNotDelete=`echo $currentFileSetCombo | cut -f2 -d':'`
        checkFilesetLevel $fileSetToDelete $fileSetLevelToNotDelete
        checkFilesetLevelRc=$?
        if (( $checkFilesetLevelRc == 1 ))
        then
          fileSetsToPreDelete="$fileSetsToPreDelete $fileSetToDelete"
          echo "Fileset is installed at a level lower than required, it will be deleted"
        elif (( $checkFilesetLevelRc == 2 ))
        then
          echo "Fileset is not installed, it does not need to be deleted"
        else
          echo "Fileset is already installed at the required level, it does not need to be deleted"
        fi
#       echo
#       echo "fileSetToDelete '$fileSetToDelete'"
#       echo "fileSetLevelToNotDelete '$fileSetLevelToNotDelete'"
#       echo "checkFilesetLevelRc '$checkFilesetLevelRc'"
#       read
      done

      # --------------------------------------------------------
      # delete filesets as required
      # --------------------------------------------------------
      echo
      if [ "$fileSetsToPreDelete" = '' ]
      then
        echo "INFO ($scriptName) - no filesets need to be pre-deleted"
      else
        theCommand="/usr/lib/instl/sm_inst installp_cmd -u -f'$fileSetsToPreDelete'"
        echo "Delete filesets with command: $theCommand"
        echo
        eval $theCommand
      fi

      # -----------------------------------------------------------
      # update the base tsm server 
      # -----------------------------------------------------------
      updateAll "$nsmTsm_update_cdrom1_tsmServer" '4-6' 'installRatherThanUpdate'

    fi

    # ------------------------------------------------------------------------
    # update TSM Server PTF 1
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipTsmServerPtf1On3466tsmUpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for TSM Server PTF 1 on 3466 Update Cdrom 1 of 2 since:"
      echo 
      echo "  skipTsmServerPtf1On3466tsmUpdateCdrom1=$skipTsmServerPtf1On3466tsmUpdateCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.tsmServerPtf1.log"
      updateAll "$nsmTsm_update_cdrom1_PTF1" '2-3'
    fi
  
    # ------------------------------------------------------
    # verify TSM Server at the proper level
    # ------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipTsmServerBaseOn3466tsmUpdateCdrom1" = 'yes' ] || [ "$skipTsmServerPtf1On3466tsmUpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping checking TSM Server level since:"
      echo 
      echo "  skipTsmClientOn3466tsmUpdateCdrom1=$skipTsmClientOn3466tsmUpdateCdrom1"
      echo "  skipTsmServerPtf1On3466tsmUpdateCdrom1=$skipTsmServerPtf1On3466tsmUpdateCdrom1"
      echo
      enterOneToContinue
    else
      echo
      echo "Checking for proper installation of TSM Server Filesets"
      echo
      reqLevel='4.1.1.0'

      checkFilesetLevel tivoli.tsm.devices.acsls $reqLevel
      checkFilesetLevelRc1=$?; 

      checkFilesetLevel tivoli.tsm.devices.fcprte $reqLevel
      checkFilesetLevelRc2=$?; 

      checkFilesetLevel tivoli.tsm.server.rte $reqLevel
      checkFilesetLevelRc3=$?; 

      checkFilesetLevel tivoli.tsm.devices.acsls $reqLevel
      checkFilesetLevelRc4=$?; 

      if (( $checkFilesetLevelRc1 == 0 )) && \
         (( $checkFilesetLevelRc2 == 0 )) && \
         (( $checkFilesetLevelRc3 == 0 )) && \
         (( $checkFilesetLevelRc4 == 0 ))
      then
        successUpdateingTsmServer='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for TSM Server was successfull"  
        echo   
      else
        echo
        echo "NSM-95341-E: ERROR verifying TSM server installation. . The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in files: /tmp/updateEc_""$sciptName/<mmdd_hhmm>.tsmServerBaseInstall.log"
        echo "                         /tmp/updateEc_""$sciptName/<mmdd_hhmm>.tsmServerPtf1.log"
        echo
        theCommand='lslpp -l | grep Tivoli'
        echo "Issuing command: $theCommand"
        echo
        eval $theCommand
        echo
        echo "  This script will exit."
        enterOneToContinue
        echo "  Exiting ....."
        unmountCd
        exit 1
        successUpdateingTsmServer='no'
      fi
    fi

    # ------------------------------------------------------------------------
    # update the webshell package
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipWebshellPackageOn3466tsmUpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for webshell package on 3466 Update Cdrom 1 of 2 since:"
      echo 
      echo "  skipWebshellPackageOn3466tsmUpdateCdrom1=$skipWebshellPackageOn3466tsmUpdateCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.webshell_package.log"
      updateAll "$nsmTsm_update_cdrom1_webshell_package" '1'
  
      # ------------------------------------------------------
      # verify Webshell at proper level
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      reqLevel='4.1.0.0'

      checkFilesetLevel adsm.StorageSmart $reqLevel
      checkFilesetLevelRc1=$?; 

      if (( $checkFilesetLevelRc1 == 0 ))
      then
        successUpdateingWebshellPackage='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for Webshell package was successfull"  
      else
        echo
        echo "NSM-95345-E: ERROR verifying Webshell package installation. The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in file: /tmp/update_ec/<mmdd_hhmm>.webshell_package.log"
        echo
        theCommand='lslpp -l adsm.StorageSmart '
        echo "Issuing command: $theCommand"
        echo 
        eval $theCommand
        echo
        echo "  This script will exit."
        enterOneToContinue
        echo "  Exiting ....."
        unmountCd
        exit 1
        successUpdateingWebshellPackage='no'
      fi
    fi

    # ------------------------------------------------------------------------
    # update the webclient package
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipWebclientPackageOn3466tsmUpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for webclient package on 3466 Update Cdrom 1 of 2 since:"
      echo 
      echo "  skipWebclientPackageOn3466tsmUpdateCdrom1=$skipWebclientPackageOn3466tsmUpdateCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.webclient_package.log"
      updateAll "$nsmTsm_update_cdrom1_webclient_package" '2-3' 'installRatherThanUpdate'
 
      # ------------------------------------------------------
      # verify webclient package installation
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      echo
      echo "Checking for proper installation of web client package"
      echo

      checkFilesetLevel adsm.aix_43                4.1.0.0
      checkFilesetLevelRc1=$?;

      checkFilesetLevel adsm.as400                 3.1.0.7
      checkFilesetLevelRc2=$?;

      checkFilesetLevel adsm.decalpha              3.7.2.0
      checkFilesetLevelRc3=$?;

      checkFilesetLevel adsm.hpux11                4.1.0.0
      checkFilesetLevelRc4=$?;

      checkFilesetLevel adsm.linux86               4.1.0.0
      checkFilesetLevelRc5=$?;

      checkFilesetLevel adsm.mac                   4.1.0.0
      checkFilesetLevelRc6=$?;

      checkFilesetLevel adsm.ncr                   3.1.0.8
      checkFilesetLevelRc7=$?;

      checkFilesetLevel adsm.nec                   3.1.0.8
      checkFilesetLevelRc8=$?;

      checkFilesetLevel adsm.netware               4.1.0.0
      checkFilesetLevelRc9=$?;

      checkFilesetLevel adsm.numaq                 4.1.0.0
      checkFilesetLevelRc10=$?;

      checkFilesetLevel adsm.os2                   3.7.2.0
      checkFilesetLevelRc11=$?;

      checkFilesetLevel adsm.sco                   4.1.0.0
      checkFilesetLevelRc12=$?;

      checkFilesetLevel adsm.sgi                   4.1.0.0
      checkFilesetLevelRc13=$?;

      checkFilesetLevel adsm.sinixr                3.1.0.8
      checkFilesetLevelRc14=$?;

      checkFilesetLevel adsm.sol267                4.1.0.0
      checkFilesetLevelRc15=$?;

      checkFilesetLevel adsm.tru64                 4.1.0.0
      checkFilesetLevelRc16=$?;

      checkFilesetLevel adsm.win32                 4.1.0.0
      checkFilesetLevelRc17=$?;

      if (( $checkFilesetLevelRc1 == 0 )) && \
         (( $checkFilesetLevelRc2 == 0 )) && \
         (( $checkFilesetLevelRc3 == 0 )) && \
         (( $checkFilesetLevelRc4 == 0 )) && \
         (( $checkFilesetLevelRc5 == 0 )) && \
         (( $checkFilesetLevelRc6 == 0 )) && \
         (( $checkFilesetLevelRc7 == 0 )) && \
         (( $checkFilesetLevelRc8 == 0 )) && \
         (( $checkFilesetLevelRc9 == 0 )) && \
         (( $checkFilesetLevelRc10 == 0 )) && \
         (( $checkFilesetLevelRc11 == 0 )) && \
         (( $checkFilesetLevelRc12 == 0 )) && \
         (( $checkFilesetLevelRc13 == 0 )) && \
         (( $checkFilesetLevelRc14 == 0 )) && \
         (( $checkFilesetLevelRc15 == 0 )) && \
         (( $checkFilesetLevelRc16 == 0 )) && \
         (( $checkFilesetLevelRc17 == 0 ))
      then
        successUpdateingWebclientPackage='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for Webshell client package was successfull"
        echo
      else
        echo
        echo "NSM-95346-E: ERROR verifying Webshell client package installation."
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in files: /tmp/updateEc_""$sciptName/<mmdd_hhmm>.webclient_package.log"
        echo
        theCommand='lslpp -l | grep adsm'
        echo "Issuing command: $theCommand"
        echo
        eval $theCommand
        echo
        echo "  This script will exit."
        enterOneToContinue
        echo "  Exiting ....."
        unmountCd
        exit 1
        successUpdateingWebclientPackage='no'
      fi
    fi

    # ------------------------------------------------------------------------
    # update TSM SWA
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipTsmSWAon3466tsmUpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping copy of TSM SWA on 3466 Update Cdrom 1 of 2 since:"
      echo 
      echo "  skipTsmSWAon3466tsmUpdateCdrom1=$skipTsmSWAon3466tsmUpdateCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.tsmSWA.log"
      updateSwaDoc 2>&1 | tee $logFileName 
    fi

    # ------------------------------------------------------------------------
    # unmount the cdrom
    # ------------------------------------------------------------------------
    unmountCd
  fi

  # --------------------------------------------------------------------------
  # Determine if 3466/TSM update cdrom  2  should be mounted
  # --------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  if [ "$skipTsmDocOn3466tsmUpdateCdrom2" = 'yes' ]
  then
    echo 
    echo "INFO ($scriptName) - skipping mounting / processing 3466/TSM Update cdrom 2 since:"
    echo 
    echo "  skipTsmDocOn3466tsmUpdateCdrom2=$skipTsmDocOn3466tsmUpdateCdrom2"
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that 3466/TSM update cdrom 2 is mounted
    # ------------------------------------------------------------------------
    fileUniqueToThisCdrom="/cd/tsmdoc.4.1.0.0/shellexc.exe"
    cdromLabel="3466 and TSM Update CD-ROM, EC: H27608 CD 2 of 2"
    if [ "$nfsHostForH27608" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$tsm_update_cdrom2_altdir"
    fi
    verifyProperCdromMounted

    # ------------------------------------------------------------------------
    # update TSM Doc
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipTsmDocOn3466tsmUpdateCdrom2" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping installing TSM 4.1.0.0 Documentation on 3466 Update Cdrom 2 since:"
      echo 
      echo "  skipTsmDocOn3466tsmUpdateCdrom2=$skipTsmDocOn3466tsmUpdateCdrom2"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.tsmDoc.log"
      updateTsmDoc 2>&1 | tee $logFileName 
    fi
  fi

} # end  updateSoftware


recordNewEcLevel ()
{
 
  InstallDate=`date +"%m%d%Y"`
  InstallTime=`date +"%H%M"`

  ECFILENAME=WebShlLvl.$InstallDate$InstallTime

  printf "$EClevelWhenDone: Filesets installed for EC level \n" > /var/adm/$ECFILENAME
  printf "Date installed is $InstallDate\n" >> /var/adm/$ECFILENAME
  lslpp -L all >> /var/adm/$ECFILENAME

  echo  
  echo "INFO: ($scriptName) - Revised EC level $EClevelWhenDone updated"
  echo
  echo "Issuing 'lsSSlevel' command to confirm"
  echo 
  lsSSlevel 
} # end recordNewEcLevel

getTsmAdminInfo ()
{
  adminInfoVerified='no'
  while [ "$adminInfoVerified" = 'no' ]
  do
    echo
    echo "*************************************************************************"
    echo "* Please enter TSM admin id that has system privileges:" 
    echo "*************************************************************************"
    read adminid
    echo
    echo "*************************************************************************"
    echo "* Please enter a valid password for $adminid:"
    echo "*************************************************************************"
    read adminpw
    echo
    echo "*************************************************************************"
    echo "Is $adminid with password $adminpw correct?"
    echo
    sleep 1
    PS3="Enter choice: "
    select userResponse in \
        'admin information is correct' \
        'admin information is NOT correct. Let me try again' \
        'abort'
    do
      case $REPLY in
        1) echo 
           echo "User responded with option 1: admin information is correct"
           checkComm='yes'
           ;;
        2) echo 
           echo "User responded with option 2: admin information is NOT correct, try again"
           checkComm='no'
           ;;
        3) echo
           echo "exiting ......"
           exit
           ;;
        *) echo
           echo "Invalid response"
           echo
           ;;
      esac
 
      if [[ -n $userResponse ]] ; then
        break
      fi

    done

    if [ "$checkComm" = 'yes' ]
    then
      echo
      theCommand="dsmadmc -id=$adminid -pa=$adminpw -server=ss-m quit"
      echo "Verifying communication with server"
      echo "Issuing command: $theCommand"
      echo
      eval $theCommand > /dev/null 2>&1
      dsmadmcRc=$?
      echo
      if (( $dsmadmcRc == 0 ))
      then
        echo "Communicated succesfully with the server"
        adminInfoVerified='yes'
        invoc="dsmadmc -id=$adminid -pa=$adminpw -server=ss-m"
      else
        echo "Communication with the server FAILED. Possible reasons are:"
        echo "  -- The userid / password may be incorrect"
        echo "  -- the server may not have had time to start fully"
        echo "  -- the shared memory stanza (ss-m) is missing or invalid"
        echo
        sleepSeconds='60'
        echo 
        echo "Sleeping for $sleepSeconds seconds to allow server time to start"
        echo
        sleep $sleepSeconds
        echo
        echo "Try again"
        echo
      fi

    fi

  done

} # end getTsmAdminInfo

updateTsmLicensing ()
{
  licenseFileName="$tsmLicenseSaveDirectory/licensedata.v41"
  echo
  echo "INFO ($scriptName): updating TSM Licensing"
  echo 

  # ------------------------------------------------------
  # load TSM license Data
  # ------------------------------------------------------
  if [ -f "$licenseFileName" ]
  then
    while read inputLine
    do
      echo "evaluating input line: $inputLine"
      eval $inputLine
    done < $licenseFileName
  else
    echo
    echo "INFO ($scriptName): unable to find licensing file: $licenseFileName"
    echo " license will be re-established with defaults. If disaster recovery"
    echo " manager or space management was licensed they will need to be"
    echo " re-licensed manually"
    echo
    enterOneToContinue
  fi 

  # ---------------------------------------------------
  # load the number of managed system lan licenses 
  # ---------------------------------------------------
  echo
  if [ "$mgsyslan_lic" = '' ]
  then
     echo "setting for mgsyslan_lic is missing. Using default value"
     mgsyslan_lic='5001'
  else
     echo "mgsyslan_lic value was loaded succesfully"
  fi
  echo "The value for mgsyslan_lic=$mgsyslan_lic"

  # ---------------------------------------------------
  # load the number of managed library licenses 
  # ---------------------------------------------------
  echo
  if [ "$library_lic" = '' ]
  then
     echo "setting for library_lic is missing. Using default value"
     library_lic='2'
  else
     echo "library_lic value was loaded succesfully"
  fi
  echo "The value for library_lic=$library_lic"

  # ---------------------------------------------------
  # load the number of space management licenses 
  # ---------------------------------------------------
  echo
  if [ "$spaceman_lic" = '' ]
  then
     echo "setting for spaceman_lic is missing. Using default value"
     spaceman_lic='0'
  else
     echo "spaceman_lic value was loaded succesfully"
  fi
  echo "The value for spaceman_lic=$spaceman_lic"

  # ---------------------------------------------------
  # load the disaster recover manager license setting
  # ---------------------------------------------------
  echo
  if [ "$drm_lic" = '' ]
  then
     echo "setting for drm_lic is missing. Using default value"
     drm_lic='No'
  else
     echo "drm_lic value was loaded succesfully"
  fi
  echo "The value for drm_lic=$drm_lic"

  # ---------------------------------------------------
  # start the server if it is not already running
  # ---------------------------------------------------
  checkIfDsmservRunning=`ps -elf | grep dsmserv | grep -v grep`
  if [ "$checkIfDsmservRunning" = '' ]
  then
    echo
    echo "INFO ($scriptName): starting TSM"
    echo 
    startadsm
    sleepSeconds='60'
    echo 
    echo "Sleeping for $sleepSeconds seconds to allow server time to start"
    echo
    sleep $sleepSeconds
  fi

  getTsmAdminInfo

  # --------------------------------------------------------------
  # Initialize managed library licenses
  # --------------------------------------------------------------
  numberMngLibAlreadyLicensed=`$invoc -commadelimit select COMPLIANCE,LIBRARY_LIC from licenses  | grep -E 'Valid|FAILED' |  cut -f 2 -d','`
  numberOfMngLibsLeftToLicense=$(($library_lic - $numberMngLibAlreadyLicensed))
  
  echo
  if (($numberOfMngLibsLeftToLicense == 0))
  then
     echo "INFO: ($scriptName) - The number of managed library licenses is $numberMngLibAlreadyLicensed"
     echo "   This is equal to the required number of managed library licenses of: $library_lic"
    elif (($numberOfMngLibsLeftToLicense < 0))
  then
     echo "NSM-95325-E: ERROR ($scriptName) - too many managed library licenses: $numberMngLibAlreadyLicensed"
     echo "   This is greater than the required number of managed library licenses of: $library_lic"
     exit 1
  else
    echo "INFO: ($scriptName) - licensing managed libraries"
    echo 
    echo "   the current number licensed is: $numberMngLibAlreadyLicensed (numberMngLibAlreadyLicensed)"
    echo "   the number of licenses required: $library_lic (library_lic)"
    echo "   the number of licenses that must be added: $numberOfMngLibsLeftToLicense (numberOfMngLibsLeftToLicense)"

    # -----------------------------------------------------
    # figure out how many license commands to issue
    # -----------------------------------------------------
    setsOf5toLic=$(($numberOfMngLibsLeftToLicense / 5))
    setsOf1toLic=$(( $numberOfMngLibsLeftToLicense - ($setsOf5toLic * 5) ))
    echo 
    echo "   the number of sets of '5'  to license: $setsOf5toLic (setsOf5toLic)"
    echo "   the number of sets of '1'  to license: $setsOf1toLic (setsOf1toLic)"

    # -----------------------------------------------------
    # issue 5library.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf5toLic > 0 ))
    then
      theCommand="$invoc register license  file=5library.lic number=$setsOf5toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95326-E: ERROR ($scriptName) - non zero return code: $commandRC creating 5library.lic"
        exit 1
      fi
    fi

    # -----------------------------------------------------
    # issue 1library.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf1toLic > 0 ))
    then
      theCommand="$invoc register license  file=1library.lic number=$setsOf1toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95327-E: ERROR ($scriptName) - non zero return code: $commandRC creating 1library.lic"
        exit 1
      fi
    fi
  fi
  
  # --------------------------------------------------------------
  # Initialize managed LAN client licenses
  # --------------------------------------------------------------
  numberMngLanClientsAlreadyLicensed=`$invoc -commadelimit select COMPLIANCE,MGSYSLAN_LIC from licenses | grep -E 'Valid|FAILED' | cut -f 2 -d','`
  numberOfMngLanClientsLeftToLicense=$(($mgsyslan_lic - $numberMngLanClientsAlreadyLicensed))

  echo
  if (($numberOfMngLanClientsLeftToLicense == 0))
  then
     echo "INFO: ($scriptName) - The number of managed LAN client licenses is $numberMngLanClientsAlreadyLicensed"
     echo "   This is equal to the required number of managed LAN client licenses of: $mgsyslan_lic"
  elif (($numberOfMngLanClientsLeftToLicense < 0))
  then
     echo "NSM-95328-E: ERROR ($scriptName) - too many managed LAN client licenses: $numberMngLanClientsAlreadyLicensed"
     echo "   This is greater than the required number of managed LAN client licenses of: $mgsyslan_lic"
     exit 1
  else
    echo "INFO: ($scriptName) - licensing managed LAN clients"
    echo
    echo "   the current number licensed is: $numberMngLanClientsAlreadyLicensed (numberMngLanClientsAlreadyLicensed)"
    echo "   the number of licenses required: $mgsyslan_lic (mgsyslan_lic)"
    echo "   the number of licenses that must be added: $numberOfMngLanClientsLeftToLicense (numberOfMngLanClientsLeftToLicense)"

    # -----------------------------------------------------
    # figure out how many license commands to issue
    # -----------------------------------------------------
    setsOf50toLic=$(($numberOfMngLanClientsLeftToLicense / 50))
    setsOf5toLic=$(( ($numberOfMngLanClientsLeftToLicense - ($setsOf50toLic * 50)) / 5 ))
    setsOf1toLic=$(( $numberOfMngLanClientsLeftToLicense - ($setsOf50toLic * 50) - ($setsOf5toLic * 5) ))
    echo
    echo "   the number of sets of '50' to license: $setsOf50toLic (setsOf50toLic)"
    echo "   the number of sets of '5'  to license: $setsOf5toLic (setsOf5toLic)"
    echo "   the number of sets of '1'  to license: $setsOf1toLic (setsOf1toLic)"

    # -----------------------------------------------------
    # issue 50mgsyslan.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf50toLic > 0 ))
    then
      theCommand="$invoc register license  file=50mgsyslan.lic number=$setsOf50toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95329-E: ERROR ($scriptName) - non zero return code: $commandRC creating 50mgsyslan.lic"
        exit 1
      fi
    fi

    # -----------------------------------------------------
    # issue 5mgsyslan.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf5toLic > 0 ))
    then
      theCommand="$invoc register license  file=5mgsyslan.lic number=$setsOf5toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95330-E: ERROR ($scriptName) - non zero return code: $commandRC creating 5mgsyslan.lic"
        exit 1
      fi
    fi

    # -----------------------------------------------------
    # issue 1mgsyslan.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf1toLic > 0 ))
    then
      theCommand="$invoc register license  file=1mgsyslan.lic number=$setsOf1toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95331-E: ERROR ($scriptName) - non zero return code: $commandRC creating 1mgsyslan.lic"
        exit 1
      fi
    fi
  fi
  
  # --------------------------------------------------------------
  # Initialize space management client licenses
  # --------------------------------------------------------------
  numberSpacemanClientsAlreadyLicensed=`$invoc -commadelimit select COMPLIANCE,SPACEMAN_LIC from licenses | grep -E 'Valid|FAILED' | cut -f 2 -d','`
  numberOfSpacemanClientsLeftToLicense=$(($spaceman_lic - $numberSpacemanClientsAlreadyLicensed))

  echo
  if (($numberOfSpacemanClientsLeftToLicense == 0))
  then
     echo "INFO: ($scriptName) - The number of space management client licenses is $numberSpacemanClientsAlreadyLicensed"
     echo "   This is equal to the required number of space managed client licenses of: $spaceman_lic"
  elif (($numberOfSpacemanClientsLeftToLicense < 0))
  then
     echo "NSM-95332-E: ERROR ($scriptName) - too many managed space management client licenses: $numberSpacemanClientsAlreadyLicensed"
     echo "   This is greater than the required number of space management client licenses of: $spaceman_lic"
     exit 1
  else
    echo "INFO: ($scriptName) - licensing space management clients"
    echo
    echo "   the current number licensed is: $numberSpacemanClientsAlreadyLicensed (numberSpacemanClientsAlreadyLicensed)"
    echo "   the number of licenses required: $spaceman_lic (spaceman_lic)"
    echo "   the number of licenses that must be added: $numberOfSpacemanClientsLeftToLicense (numberOfSpacemanClientsLeftToLicense)"

    # -----------------------------------------------------
    # figure out how many license commands to issue
    # -----------------------------------------------------
    setsOf50toLic=$(($numberOfSpacemanClientsLeftToLicense / 50))
    setsOf5toLic=$(( ($numberOfSpacemanClientsLeftToLicense - ($setsOf50toLic * 50)) / 5 ))
    setsOf1toLic=$(( $numberOfSpacemanClientsLeftToLicense - ($setsOf50toLic * 50) - ($setsOf5toLic * 5) ))
    echo
    echo "   the number of sets of '50' to license: $setsOf50toLic (setsOf50toLic)"
    echo "   the number of sets of '5'  to license: $setsOf5toLic (setsOf5toLic)"
    echo "   the number of sets of '1'  to license: $setsOf1toLic (setsOf1toLic)"

    # -----------------------------------------------------
    # issue 50spacemgr.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf50toLic > 0 ))
    then
      theCommand="$invoc register license  file=50spacemgr.lic number=$setsOf50toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95333-E: ERROR ($scriptName) - non zero return code: $commandRC creating 50spacemgr.lic"
        exit 1
      fi
    fi

    # -----------------------------------------------------
    # issue 5spacemgr.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf5toLic > 0 ))
    then
      theCommand="$invoc register license  file=5spacemgr.lic number=$setsOf5toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95334-E: ERROR ($scriptName) - non zero return code: $commandRC creating 5spacemgr.lic"
        exit 1
      fi
    fi

    # -----------------------------------------------------
    # issue 1spacemgr.lic commands as required
    # -----------------------------------------------------
    if (( $setsOf1toLic > 0 ))
    then
      theCommand="$invoc register license  file=1spacemgr.lic number=$setsOf1toLic"
      echo
      echo "INFO: ($scriptName) - issuing command: $theCommand"
      eval $theCommand
      commandRc=$?
      if (( $commandRc > 0  ))
      then
        echo "NSM-95335-E: ERROR ($scriptName) - non zero return code: $commandRC creating 1spacemgr.lic"
        exit 1
      fi
    fi
  fi

  # -----------------------------------------------------
  # init DRM licensing
  # -----------------------------------------------------
  if [ "$drm_lic" = 'Yes' ]
  then
    theCommand="$invoc register license file=drm.lic"
    echo
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
    commandRc=$?
    if (( $commandRc > 0  ))
    then
      echo "NSM-95336-E: ERROR ($scriptName) - non zero return code: $commandRC creating drm.lic"
      exit 1
    fi
  else
    echo
    echo "TSM disaster recovery manager is not licensed"
  fi

  # -----------------------------------------------------
  # display licensing
  # -----------------------------------------------------
  theCommand="$invoc select \* from licenses" 
  echo
  echo "INFO: ($scriptName) - issuing command: $theCommand"
  eval $theCommand

} # end updateTsmLicensing

phase1CaptureTsmlic ()
{
  echo
  echo "INFO ($scriptName): entering phase 1 - capture TSM 3.7 licensing"
  echo
  checkIfDsmservRunning=`ps -elf | grep dsmserv | grep -v grep`
  if [ "$checkIfDsmservRunning" = '' ]
  then
    echo
    echo "INFO ($scriptName): starting TSM"
    echo 
    startadsm
    sleepSeconds='60'
    echo 
    echo "Sleeping for $sleepSeconds seconds to allow server time to start"
    echo
    sleep $sleepSeconds
  fi

  getTsmAdminInfo

  # ---------------------------
  # TSM Administrator Variables
  # ---------------------------

  adminID=$adminid
  adminPW=$adminpw

  # --------
  # EC Level
  # --------

  ecLevel="$requiredPrereqEC"
  ecLevelWhenComplete="$EClevelWhenDone"

  # --------------------
  # License output files
  # --------------------

  v37lic="$tsmLicenseSaveDirectory/licensedata.v37"
  v41lic="$tsmLicenseSaveDirectory/licensedata.v41"

  # -------------------------------------------------------------------------
  # Checks to make sure the customer is at v3r7 of the TSM server.  Fails out
  # if any other version is detected, and reports that version.
  # -------------------------------------------------------------------------

  echo
  echo "INFO: ($scriptName) - Checking to see which version of TSM is currently"
  echo "                      installed ..."

  whatVersion=`dsmadmc -id=$adminID -pa=$adminPW -server=ss-m q st | grep 'Storage Management Server' | awk '{print $8 $10}' | cut -b1,3`

  if [ "$whatVersion" = '37' ]
  then
    echo
    echo "INFO: ($scriptName) - TSM for AIX Version 3, Release 7 detected."
  else
    tsmVersion=`echo $whatVersion | cut -b1`
    tsmRelease=`echo $whatVersion | cut -b2`
    doesThisEcExist=`lsSSlevel | grep H27349`

    if [ "$doesThisEcExist" = '' ]
    then
      echo
      echo "WARNING: ($scriptName) - TSM for AIX Version $tsmVersion, Release $tsmRelease is already installed,"
      echo "                         but EC $ecLevelWhenComplete has NOT been applied!  If you have not"
      echo "                         manually licensed your server via the TSM command"
      echo "                         'reg lic file(filename.lic)', the following default"
      echo "                         licenses will be issued for your v"$tsmVersion"r$tsmRelease server:"
      echo
      echo "                         * 5001 Managed System for LAN clients"
      echo "                         * 2 Managed Libraries"
      return $programRC=1
    else
      echo
      echo "INFO: ($scriptName) - TSM for AIX Version $tsmVersion, Release $tsmRelease is already installed."
      echo "                      The command 'lsSSlevel' indicates that EC $ecLevelWhenComplete has"
      echo "                      previously been applied."
      return $programRC=1
    fi
  fi

  # -----------------------------------------------------------------------
  # Check to see if one or all of the following 3 unsupported protocols for
  # v4r1 is in use on the v3r7 server.
  # -----------------------------------------------------------------------

  anyCommMethodsInUse='no'
  snaInUse=`dsmadmc -id=$adminID -pa=$adminPW "q opt" | grep "SNAlu6.2"`
  ipxspxInUse=`dsmadmc -id=$adminID -pa=$adminPW "q opt" | grep "IPXSPX"`
  netbiosInUse=`dsmadmc -id=$adminID -pa=$adminPW "q opt" | grep "NETBIOS"`

  if [ "$snaInUse" != '' ]
  then
    anyCommMethodsInUse='yes'
    snaName='SNAlu6.2'
  fi
  if [ "$ipxspxInUse" != '' ]
  then
    anyCommMethodsInUse='yes'
    ipxspxName='IPXSPX'
  fi
  if [ "$netbiosInUse" != '' ]
  then
    anyCommMethodsInUse='yes'
    netbiosName='NETBIOS'
  fi

  echo
  echo
  echo "INFO: ($scriptName) - Checking current server configuration to see if any of"
  echo "                      the following communications methods are in use:"
  echo
  echo "                      SNAlu6.2"
  echo "                      IPXSPX"
  echo "                      NETBIOS"
  echo
  echo "                      These protocols are no longer supported in v4r1 of the"
  echo "                      TSM server.  If any of them are detected as in use, you"
  echo "                      should remove them from the server options file.  If"
  echo "                      the server is started in the foreground for any reason,"
  echo "                      \"Invalid keyword\" error messages will be shown, but"
  echo "                      the server activity log will not record these messages."

  if [ "$anyCommMethodsInUse" = 'yes' ]
  then
    echo
    echo "Press [Enter] to continue:"
    read
  fi

  if [ "$anyCommMethodsInUse" = 'no' ]
  then
    echo
    echo "INFO: ($scriptName) - None of the communications methods listed above were"
    echo "                      detected.  Proceeding normally."
  else
    echo
    echo "WARNING: ($scriptName) - The following communications methods are in use:"
    echo

    if [ "$snaName" != '' ]
    then
      echo "                         $snaName"
    fi
    if [ "$ipxspxName" != '' ]
    then
      echo "                         $ipxspxName"
    fi
    if [ "$netbiosName" != '' ]
    then
      echo "                         $netbiosName"
    fi

    echo
    echo "                         These protocols are no longer supported in v4r1 of the"
    echo "                         TSM server.  If you continue, some clients may not be"
    echo "                         able to communicate with the server.  You must change"
    echo "                         those clients' options files to use either TCP-IP, or"
    echo "                         Shared Memory (if the client is located on the same"
    echo "                         system as the TSM server), or use a web browser if"
    echo "                         the previous suggestions are not viable options."
    echo
    echo "                         The dsmserv.opt file in the TSM server directory must"
    echo "                         be edited to remove those keywords listed above."
    echo "                         Usually the word \"COMMmethod\" will preceed each"
    echo "                         protocol keyword."
    echo
    echo "Press [Enter] to continue:"
    read
  fi

  # --------------------------------------------------------------------------
  # Display some introductory text as to why this licensing conversion process
  # is required.
  # --------------------------------------------------------------------------

  echo
  echo "INFO: ($scriptName) - The following procedure captures the licensing"
  echo "                      information from your TSM v3r7 server and translates"
  echo "                      this data to their v4r1 equivalents.  This process is"
  echo "                      necessary due to a new licensing structure introduced"
  echo "                      in v4 of TSM, and the lack of an automatic conversion"
  echo "                      process in the TSM software itself.  The following"
  echo "                      steps will be performed to facilitate this transition:"
  echo
  echo "                      <1> The v3r7 TSM server will be queried for currently"
  echo "                          licensed features.  This information will then be"
  echo "                          saved to the following file:"
  echo
  echo "                          $v37lic"
  echo
  echo "                      <2> Afterwards, based on the information captured above,"
  echo "                          the converted v4r1 license data will be saved to the"
  echo "                          following file:"
  echo
  echo "                          $v41lic"
  echo
  echo
  echo "Press [Enter] to continue:"
  read
  echo
  echo "                      <3> Because the licensing structure is quite different,"
  echo "                          some existing v3r7 licenses do not readily translate"
  echo "                          to a v4r1 counterpart.  Therefore at the end of this"
  echo "                          procedure a report will be generated listing the"
  echo "                          licenses which have been enabled, and also those"
  echo "                          which require manual intervention (if any)."
  echo
  echo "                      Notes:"
  echo
  echo "                      <a> If the information was previously captured and the"
  echo "                          license data is determined to be good, then the"
  echo "                          existing files mentioned above will be used, and the"
  echo "                          server will not queried again."
  echo
  echo "                      <b> If the captured data is bad (ie., due to this script"
  echo "                          being cancelled for any reason (IMPORTANT: and before"
  echo "                          this EC is applied!), then the pre-existing files"
  echo "                          will be removed and the license information"
  echo "                          re-captured."
  echo
  echo
  echo "Press [Enter] to continue:"
  read

  # ---------------------------------------------------------------------
  # Checks to make sure the /adsmlog filesystem is mounted so the license
  # output can be written to a file.
  # ---------------------------------------------------------------------

  echo
  echo "INFO: ($scriptName) - Making sure /adsmlog is available so the license data"
  echo "                      can be output to files ..."
  isItMounted=`df /adsmlog | grep adsmlog | awk '{print $7}'`

  if [ "$isItMounted" = "/adsmlog" ]
  then
    echo
    echo "INFO: ($scriptName) - /adsmlog is mounted."
  else
    echo
    echo "NSM-95337-E: ERROR: ($scriptName) - The /adsmlog filesystem is not mounted!  Please correct"
    echo "                       the condition and restart this phase."
    return $programRC=1
  fi

  # -------------------------------------------------------------------------
  # Checks to see if there is an existing v3r7 license capture file.  If so,
  # check to see if it is valid (capturecomplete=yes).  If not, delete it and
  # re-capture.  If the file never existed, proceed with the capture.
  # -------------------------------------------------------------------------

  if [[ -d "$tsmLicenseSaveDirectory" ]]
  then
    if [[ -f $v37lic ]]
    then
      echo
      echo "INFO: ($scriptName) - Existing captured v3r7 license data was found in"
      echo "                      $v37lic."
      echo
      echo "INFO: ($scriptName) - Checking $v37lic to make sure it"
      echo "                      contains valid information."
      v37cap=`cat $v37lic | grep 'capturecompleted=yes'`
      if [ "$v37cap" != '' ]
      then
        echo
        echo "INFO: ($scriptName) - $v37lic contains valid v3r7"
        echo "                      license capture data."
        if [[ -f $v41lic ]]
        then
          echo
          echo "INFO: ($scriptName) - Existing v4r1 license data was found in"
          echo "                      $v41lic."
          echo
          echo "INFO: ($scriptName) - Checking $v41lic to make sure it"
          echo "                      contains valid information."
          v41cap=`cat $v41lic | grep 'outputcompleted=yes'`
          if [ "$v41cap" != '' ]
          then
            echo
            echo "INFO: ($scriptName) - $v41lic contains valid v4r1"
            echo "                      license output data."
            doINeedToCapture='no'
          else
            echo
            echo "WARNING: ($scriptName) - Invalid license data found in $v41lic."
            echo "                         File will be removed.  Proceeding with capture."
            rm $v37lic 2> /dev/null
            rm $v41lic
            doINeedToCapture='yes'
          fi
        fi
      else
        echo
        echo "WARNING: ($scriptName) - Invalid license data found in $v37lic."
        echo "                         File will be removed.  Proceeding with capture."
        rm $v37lic
        rm $v41lic 2> /dev/null
        doINeedToCapture='yes'
      fi
    else
      echo
      echo "INFO: ($scriptName) - No license data file was found.  Proceeding with"
      echo "                      license capture to $v37lic."
      rm $v41lic 2> /dev/null
      doINeedToCapture='yes'
    fi
  else
    mkdir $tsmLicenseSaveDirectory
    echo
    echo "INFO: ($scriptName) - No license data file was found.  Proceeding with"
    echo "                      license capture to $v37lic."
   doINeedToCapture='yes'
  fi

  # -----------------------------------------------------------------------
  # If no previous license data file was detected or a bad one was detected
  # (and then deleted, then new license information is captured.
  # -----------------------------------------------------------------------

  if [ "$doINeedToCapture" = 'yes' ]
  then
    echo 
    echo "INFO: ($scriptName) - Querying the TSM Server for currently licensed"
    echo "                      features ..."

    # Checking server license compliance
    #
    compliance=`dsmadmc -id=$adminID -pa=$adminPW -commadelimit 'select compliance from licenses' | grep -E "Valid|Invalid"`

    if [ "$compliance" != "Valid" ]
    then
      echo
      echo "NSM-95338-E: ERROR: ($scriptName) - TSM Server licensing is not in compliance! Contact your"
      echo "                       Tivoli service representative to resolve this issue."
      echo
      return $programRC=1
    fi

    # -----------------------------------------------------------------------
    # Goes through all the license fields to determine their current state on
    # the v3r7 server.  The following list are captured fields:
    #
    # License                                     SQL Field
    #
    # Licensed client nodes                       (nodes_lic)
    # Network connections licensed                (network_lic)
    # Open Systems Environment clients licensed   (opensys_lic)
    # Space Management licensed                   (spaceman_lic)
    # Tivoli Disaster Recovery Manager licensed   (drm_lic)
    # Advanced Device Support licensed            (advdev_lic)
    # TDP for Oracle licensed                     (oracle_lic)
    # TDP for MS SQL Server licensed              (mssql_lic)
    # TDP for MS Exchange licensed                (msexch_lic)
    # TDP for Lotus Notes licensed                (lnotes_lic)
    # TDP for Lotus Domino licensed               (domino_lic)
    # TDP for Informix licensed                   (informix_lic)
    # TDP for SAP R/3 licensed                    (sapr3_lic)
    # -----------------------------------------------------------------------

    v3r7Licenses="nodes_lic network_lic opensys_lic spaceman_lic drm_lic advdev_lic oracle_lic mssql_lic msexch_lic lnotes_lic domino_lic informix_lic sapr3_lic"
    for currentLicense in $v3r7Licenses
    do
      currentLicenseValue=`dsmadmc -id=$adminID -pa=$adminPW -commadelimit "select $currentLicense,compliance from licenses" | grep -E "Valid|Invalid" | cut -f1 -d','`
      echo "$currentLicense=$currentLicenseValue" >> $v37lic
      eval "$currentLicense=$currentLicenseValue"
    done

    # Put a stamp in to reflect the data was captured successfully
    #
    echo "capturecompleted=yes" >> $v37lic
    echo 
    echo "INFO: ($scriptName) - v3r7 license information was gathered successfully and"
    echo "                      output to $v37lic."

    # Output the v4r1 license data file based on input from the v3r7 SQL queries
    #
    echo "mgsyslan_lic=5001" >> $v41lic

    if [ "$spaceman_lic" = 'Yes' ]
    then
      echo "spaceman_lic=250" >> $v41lic
    else
      echo "spaceman_lic=0" >> $v41lic
    fi

    if [ "$drm_lic" = 'Yes' ]
    then
      echo "drm_lic=Yes" >> $v41lic
    else
      echo "drm_lic=No" >> $v41lic
    fi

    echo "library_lic=2" >> $v41lic

    # Put a stamp in to reflect the data was output successfully
    #
    echo "outputcompleted=yes" >> $v41lic
    echo 
    echo "INFO: ($scriptName) - v4r1 license information was converted successfully and"
    echo "                      output to $v41lic."
  fi

  # --------------------------------------------------------------------------
  # This routine reads the license data from the v3r7 license file and reports
  # what the customer has, and warns if any licenses cannot be migrated to
  # v4r1.  This is needed because the capture won't always be run from scratch
  # (ie., if a good license output file was detected already).
  # --------------------------------------------------------------------------

  while read -r line
  do
    currentLicense=`echo $line | cut -f1 -d'='`
    currentLicenseValue=`echo $line | cut -f2 -d'='`
    eval "$currentLicense=$currentLicenseValue"
  done < $v37lic

  # -------------------------------
  # Collect TSM and AIX server info
  # -------------------------------

  tsmServername=`dsmadmc -id=$adminID -pa=$adminPW -commadelimit select server_name,licensecompliance from status | grep -E "Valid|Invalid" | cut -f1 -d','`
  aixHostname=`hostname`

  echo
  echo "INFO: ($scriptName) - Ready to display gathered licensing information ..."

  echo
  echo
  echo "Press [Enter] to continue:"
  read

#  echo
#  echo "[!] - TSM Server \"$tsmServername\" on AIX host \"$aixHostname\":"
#  echo

  echo
  echo "INFO: ($scriptName) - The following TSM v3r7 features are licensed:"
  echo

  if [[ "$nodes_lic" > 1 ]]
  then
    echo "[*] - $nodes_lic licensed client nodes were detected."
  else
    echo "[*] - $nodes_lic licensed client node was detected."
  fi

  if [ "$network_lic" = "Yes" ]
  then
    echo "[*] - Network connections are licensed.                                  "
    displayWarning='no'
  fi
  if [ "$opensys_lic" = "Yes" ]
  then
    echo "[*] - Open Systems Environment clients are licensed.                     "
    displayWarning='no'
  fi
  if [ "$spaceman_lic" = "Yes" ]
  then
    echo "[*] - Space Management is licensed."
  fi
  if [ "$drm_lic" = "Yes" ]
  then
    echo "[*] - Tivoli Disaster Recovery Manager is licensed."
  fi
  if [ "$advdev_lic" = "Yes" ]
  then
    echo "[*] - Advanced Device Support is licensed.                               "
    displayWarning='no'
  fi
  if [ "$oracle_lic" = "Yes" ]
  then
    echo "[?] - TDP for Oracle is licensed.                        (See Note Below)"
    displayWarning='yes'
  fi
  if [ "$mssql_lic" = "Yes" ]
  then
    echo "[?] - TDP for MS SQL Server is licensed.                 (See Note Below)"
    displayWarning='yes'
  fi
  if [ "$msexch_lic" = "Yes" ]
  then
    echo "[?] - TDP for MS Exchange is licensed.                   (See Note Below)"
    displayWarning='yes'
  fi
  if [ "$lnotes_lic" = "Yes" ]
  then
    echo "[?] - TDP for Lotus Notes is licensed.                   (See Note Below)"
    displayWarning='yes'
  fi
  if [ "$domino_lic" = "Yes" ]
  then
    echo "[?] - TDP for Lotus Domino is licensed.                  (See Note Below)"
    displayWarning='yes'
  fi
  if [ "$informix_lic" = "Yes" ]
  then
    echo "[?] - TDP for Informix is licensed.                      (See Note Below)"
    displayWarning='yes'
  fi
  if [ "$sapr3_lic" = "Yes" ]
  then
    echo "[?] - TDP for SAP R/3 is licensed.                       (See Note Below)"
    displayWarning='yes'
  fi

  if [ "$displayWarning" = "yes" ]
  then
    echo
    echo "NOTE: For licenses which show a [?], you must contact Tivoli support to"
    echo "      determine an upgrade path.  There is no direct update from v3r7 to"
    echo "      v4r1, therefore the TSM command 'reg lic file(filename.lic)' must"
    echo "      be used in v4r1 to enable the equivalent feature in v4r1."
  fi

  echo
  echo "Press [Enter] to continue:"
  read

  if [[ "$nodes_lic" -gt 5001 ]]
  then
    echo
    echo "WARNING: ($scriptName) - More than 5001 client nodes are registered on this"
    echo "                         system.  Your v4r1 TSM server will only be licensed"
    echo "                         for 5001 Managed System for LAN clients.  Please"
    echo "                         contact Tivoli support if more licenses are desired"
    echo "                         in this category."

    echo
    echo
    echo "Press [Enter] to continue:"
    read
  fi

  # --------------------------------------------------------------------------
  # This routine reads the license data from the v4r1 license file and reports
  # what the customer will be getting.  Only those items which have previously
  # been reported as migratable (besides the default licenses) will be
  # reported.  This is needed because the capture won't always be run from
  # scratch (ie., if a good license output file was detected already).
  # --------------------------------------------------------------------------

  while read -r line
  do
    currentLicense=`echo $line | cut -f1 -d'='`
    currentLicenseValue=`echo $line | cut -f2 -d'='`
    eval "$currentLicense=$currentLicenseValue"
  done < $v41lic

  echo
  echo "INFO ($scriptName) - The following features will be licensed on the v4r1"
  echo "                     server once the TSM upgrade is complete:"

  echo
  echo "[*] - $mgsyslan_lic Managed System for LAN clients will be licensed."

  if [[ $spaceman_lic = 250 ]]
  then
    echo "[*] - $spaceman_lic Space Management clients will be licensed."
  fi

  if [ "$drm_lic" = 'Yes' ]
  then
    echo "[*] - Tivoli Disaster Recovery Manager will be licensed."
  fi

  echo "[*] - $library_lic Managed Libraries will be licensed."

  # -------------------------------------------------------------------------
  # Display final warning message about some licenses not able to be migrated
  # if any were detected previously.
  # -------------------------------------------------------------------------

  if [ "$displayWarning" = "yes" ]
  then
    echo
    echo "WARNING: ($scriptName) - Reminder: Since licensed features [?] were previously"
    echo "                         detected which do not have a direct migration path"
    echo "                         from v3r7 to v4r1, you need to contact Tivoli support"
    echo "                         for further assistance in obtaining the correct"
    echo "                         equivalent licenses, and directions for enabling them"
    echo "                         using the 'reg lic file(filename.lic)' TSM command."
  fi

  if [[ "$nodes_lic" -gt 5001 ]]
  then
    echo
    echo
    echo "Press [Enter] to continue:"
    read

    echo
    echo "WARNING: ($scriptName) - Reminder: More than 5001 licensed clients have been"
    echo "                         detected on your v3r7 TSM server.  Any client"
    echo "                         licenses beyond 5001 will not be migrated to v4r1."
    echo "                         Please contact Tivoli support if more licenses are"
    echo "                         desired in this category."
  fi

  # --------------------------------------------------
  # Display exit message from license capture routine.
  # --------------------------------------------------

  echo
  echo "INFO: ($scriptName) - TSM v3r7 licensing capture procedure is complete."

} # end phase1CaptureTsmlic

endTheTsmServer ()
{
  allServersDown='no'
  while [ "$allServersDown" = 'no' ]
  do
    checkIfDsmservRunning=`ps -elf | grep dsmserv | grep -v grep`
    if [ "$checkIfDsmservRunning" != '' ]
    then
      echo
      echo "The TSM server is running:"
      echo 
      ps -elf | grep dsmserv | grep -v grep 
      echo
      echo "Hit '1' to continue and bring the server down"
      echo 
      enterOneToContinue
      endadsm
    else
      allServersDown='yes'
      echo 
      echo "INFO ($scriptName): verified all servers are down"
    fi
  done
} # end endTheTsmServer 

phase2UpgradeSystem ()
{
  # ------------------------------------------------------------------------
  # Bring server down if required
  # ------------------------------------------------------------------------
  if [ "$skipBringingServerDown" = 'yes' ]
  then
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "Skipping bringing the TSM server down"
    echo
    enterOneToContinue
  else
    endTheTsmServer
  fi

  # ------------------------------------------------------------------------
  # Verify filesystem size
  # ------------------------------------------------------------------------
  if [ "$skipUsrFilesystemAdjustment" = 'yes' ]
  then
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "Skipping adjustment of the /usr filesystem size"
    echo
    enterOneToContinue
    echo
  else
    adjustSizeOfUsrFilesystem
  fi 

  # ------------------------------------------------------------------------
  # create the cdrom filesystem
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  mkdir /cd >/dev/null 2>&1
  echo 
  str=`cat /etc/filesystems | grep /dev/cd0`
  if [ "$str" = '' ] 
  then
    echo "INFO: ($scriptName) - Creating CDROM filespace /cd."
    theCommand="crfs -v cdrfs -d /dev/cd0 -m /cd -p ro"
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
    crfsRc=$?
    if [ "$crfsRc" != 0 ]
    then
      echo "NSM-95339-E: ERROR: ($scriptName) Unable to create cdrom filesystem."
      echo "Resolve problem and repeat this script."
      exit 1
    fi
  else
    echo "INFO: ($scriptName) - CDROM filespace /cd already exists"
  fi

  # ------------------------------------------------------------------------
  # cleanup just in case
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  theCommand="installp -C" 
  echo "INFO: issuing command: $theCommand in case a previous installp"
  echo "      failed and requires cleanup"
  echo 
  eval $theCommand

  # ------------------------------------------------------
  # update software
  # ------------------------------------------------------
  updateSoftware

  # ------------------------------------------------------
  # update TSM licensing
  # ------------------------------------------------------
  if [ "$skipServerRelicense" = 'yes' ]
  then
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "INFO ($scriptName) Skipping server re-licensing"
    echo
    enterOneToContinue
    echo
  else
    updateTsmLicensing
  fi
  
  # ------------------------------------------------------
  # echo final status
  # ------------------------------------------------------
  echo "SUCCESS: ($scriptName) all updates complete"
  echo
  echo
  echo "----------------------------------------------------------------------"
  echo
  if [ "$isNewEcApplied" = '' ]
  then
    recordNewEcLevel
  else
    echo
    echo "The EC $EClevelWhenDone already recorded as complete"
  fi 
 
  theCommand='lsSSlevel'
  echo
  echo "Issuing command: $theCommand"
  echo 
  eval $theCommand
  
  # ------------------------------------------------------------------------
  # unmount the /cd filesystem 
  # ------------------------------------------------------------------------
  theCommand='unmount /cd'
  echo
  echo "Issuing command: $theCommand"
  echo
  eval $theCommand
  theCommandRc=$?
  echo
  if (( $theCommandRc == 0 ))
  then
    echo "The cdrom was unmounted."
    echo "Remove the cdrom from the reader" 
  else
    echo "NSM-95349-E: ERROR unmounting cdrom. Manual intervention required"
    echo
    echo '  1) determine why cdrom cannot be unmounted'
    echo '     one likely cause is the directory is cded into'
    echo '     from some other window'
    echo "  2) manually issue the command: '$theCommand'"
    echo "  3) Once the /cd file system is unmounted, remove the cdrom from the reader"
  fi
  enterOneToContinue

  # ------------------------------------------------------------------------
  # remove the /cd filesystem 
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  echo "INFO: ($scriptName) - removing /cd filesystem from /etc/filesystems"
  theCommand='rmfs /cd'
  echo "INFO: ($scriptName) - issuing command: $theCommand"
  eval $theCommand
  rmfsRc="$?"
  if [ $rmfsRc != 0 ]
  then
    echo "NSM-95350-E: ERROR: ($scriptName) - Unable to remove the /cd filesystem"
  fi

  # ----------------------------------------------------------------------
  # Bring the TSM server(s) down
  # ----------------------------------------------------------------------
    endTheTsmServer

  # ----------------------------------------------------------------------
  # prompt user to reboot the system
  # ----------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  echo "INFO: ($scriptName) - issue 'shutdown -Fr' to reboot the system"
  enterOneToContinue

} # end phase2UpgradeSystem

 
# ************************************************************************

# -------------------------------------------------------------------------
# Process special options.
#   1) Special options must start with a '-'
#   2) The special option must be adjacent to the '-' and can be any
#      number of characters
#   3) Any number of values can be specified for a special option.
#      If values are processed they must be captured and then shifted left
#   4) All special options must be specified prior to positional parameters
# -------------------------------------------------------------------------

firstCharParm1=`echo $1 | cut -b1`
while [ "$firstCharParm1" = '-' ]
do
  specialOption=`echo $1 | cut -b2-`
  case $specialOption in
    ('h')
      helpOption='yes'
      : ;;
    ('b')
      bypassSSlevelCheck='yes'
      : ;;
    (*)
      echo
      echo "NSM-95352-E: ERROR: ($scriptName) - special option: '$1' not recognized"
      echo "  program terminated"
      exit       
      : ;;
  esac
  shift 1
  firstCharParm1=`echo $1 | cut -b1`
done

# -----------------------------------------------------------------------------
# Write help text as appropriate
# -----------------------------------------------------------------------------
tempHelpFile='/tmp/tempHelpFile'
if [ "$helpOption" = 'yes' ]
then
  echo > $tempHelpFile
  echo "--- Hit SPACE key to move forward one screen" >> $tempHelpFile
  echo "--- Hit 'b' to move backward one screen" >> $tempHelpFile
  echo "--- Hit ENTER to move forward one line" >> $tempHelpFile
  echo  >> $tempHelpFile
  cat $0 | grep \#@ | grep -v 'grep #@'} | cut -b3- >> $tempHelpFile

  clear
  cat $tempHelpFile | grep -v tempHelpFile | more  

  exit
fi

mainRoutine ()
{

  # ------------------------------------------------------------------------
  # say hello
  # ------------------------------------------------------------------------
  echo
  echo "INFO: ENTERING ($scriptName) on `date`"
  echo 

  # ------------------------------------------------------------------------
  # display control variables as appropriatte
  # ------------------------------------------------------------------------
  displayControlVariables

  # ------------------------------------------------------------------------
  # initialize control variables
  # ------------------------------------------------------------------------
  initializeInstallationDirectories

  echo
  echo "----------------------------------------------------------------------"
  echo
  # ----------------------------------------------------------
  # Make sure the system is an RS/6000 H50           
  # ----------------------------------------------------------
  str=$(uname -m | cut -c 9,10 )
  if [ "$str" = "4C" ]
  then
     str=`lsattr -El sys0 | grep modelname | grep H50`
     if [ "$str" = '' ]
     then
       systemIsAnH50='no'
     else
       systemIsAnH50='yes'
     fi
  else
    systemIsAnH50='no'
  fi

  echo
  if [ "$systemIsAnH50" = 'yes' ]
  then
    echo "INFO: ($scriptName) - confirmed that system is an RS/6000 H50"
  else
    echo "NSM-95353-E: ERROR: ($scriptName) - system is not an RS/6000 H50"
    echo 
    theCommand='lsattr -El sys0 | grep modelname | grep H50'
    echo "Issuing command: $theCommand"
    echo 
    eval $theCommand
    echo 
    echo "exiting ....."
  fi

  # ------------------------------------------------------------------------
  # Make sure the system has applied the required pre-requisite EC H27349
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  requiredPrereqEC='H27349'
  if [ "$bypassSSlevelCheck" = 'yes' ]
  then
    echo
    echo "INFO: ($scriptName) - skipping check for pre-requisite EC $requiredPrereqEC"
    echo "       per user input"
  else 
    isRequiredEcApplied=`lsSSlevel | grep $requiredPrereqEC` 
    echo
    if [ "$isRequiredEcApplied" = '' ]
    then
      echo
      echo "NSM-95354-E: ERROR: ($scriptName) - required EC $requiredPrereqEC not applied"
      echo "  exiting ...."
      exit
    else
      echo
      echo "INFO: ($scriptName) - verified that required EC $requiredPrereqEC is applied"
    fi
  fi

  # ------------------------------------------------------------------------
  # Set the name for the TSM license save directory
  # ------------------------------------------------------------------------
  tsmLicenseSaveDirectory="/adsmlog/tsmLicenseInfoForEcH27608"

  # ------------------------------------------------------------------------
  # See if the new EC already is applied
  # ------------------------------------------------------------------------
  EClevelWhenDone='H27608a'
  requiredPrereqEC='H27349'
  isNewEcApplied=`lsSSlevel | grep $EClevelWhenDone` 
  echo
  if [ "$isNewEcApplied" = '' ]
  then
    :
  else
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "INFO: ($scriptName) - the 'lsSSlevel' command indicates that"
    echo "   EC $EClevelWhenDone is already applied"
    echo
    echo "Indicate what action to take"
    echo
    sleep 1
    PS3="Enter choice: "
    select userResponse in \
        'continue - apply the EC again' \
        'exit'
    do
      case $REPLY in
        1) :
           ;;
        2) echo
           echo "exiting ......"
           exit
           ;;
        *) echo 'Invalid Selection.'
           ;;
      esac
 
      if [[ -n $userResponse ]] ; then
        break
      fi
    done
  fi


  # ------------------------------------------------------------------------
  # prompt user
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  sleep 1 # seems to be required, or output gets messed up
  PS3="
Enter choice: "
  select userResponse in \
      'phase 1 - capture TSM 3.7 Licensing'\
      'phase 2 - upgrade the system'\
      'abort'
  do
    case $REPLY in
      1) echo
         echo "User choice option 1: capture TSM 3.7 Licensing"
         echo
         phase1CaptureTsmlic
         ;;
      2) echo
         echo "User choice option 2: upgrade the system"
         echo
         phase2UpgradeSystem
         ;;
      3) echo
         echo "exiting ......"
         exit
         ;;
      *) echo "Invalid selection"
         ;;
    esac

    if [[ -n $userResponse ]] ; then
      break
    fi
  done

  # ------------------------------------------------------------------------
  # say goodbye
  # ------------------------------------------------------------------------
  echo
  echo "INFO: EXITING ($scriptName) on `date`"
  echo

} # end mainRoutine

# ------------------------------------------------
# Set up logging environement
# ------------------------------------------------
export scriptName='h27608a3.ec'
buildID=`date "+%m%d_%H%M"`
updateLogDir="/tmp/updateEc_""$scriptName"
mkdir -p $updateLogDir >/dev/null 2>&1
consoleOutputFile="$updateLogDir/$buildID.consoleLog"

# ------------------------------------------------
# Check for special cdrom option
# ------------------------------------------------
if [ "$nfsHostForH27608" = '' ]
then
  :
else
  echo
  echo "**********************************************************************"
  echo "*"
  echo "* A special development override is specified to simulate cdroms"
  echo "*   on the following file server: $nfsHostForH27608"
  echo "*"
  echo "**********************************************************************"
  echo
  enterOneToContinue
  crfs -v cdrfs -d /dev/cd0 -m /cd -p ro >/dev/null 2>&1 # add it can be removed
fi

# ------------------------------------------------
# Invoke the main routine to do the work
# ------------------------------------------------
clear
echo
echo "----------------------------------------------------------------------"
echo
echo "INFO: ($scriptName): console output is logged in the file: $consoleOutputFile" 
echo

# ----------------------------------------------------------------
# set trap for CNTL-C and execute main routine
# ----------------------------------------------------------------
trap cntlCtrap INT # unmount /cd and remove the cdrom ftrap
mainRoutine | tee $consoleOutputFile 2>&1
